
#-----------------------------------------------------------------------------
# Name        	: SkIndigo_08.rb
# Description 	: Model exporter and material editor for Indigo 0.8 Stable
# Menu Item   	: Plugins\SkIndigo\
# Author      	: Dale Martens (Whaat)
#		  Code used from following SU exporters: Su2KT by Tomasz, SU2POV by Didier Bur and OGRE exporter by Kojack
# Usage     	: Copy script to PLUGINS folder in SketchUp folder, run SU, go to Plugins\SkIndigo
# Date        	: XX/XX/2007
# Type        	: Exporter
# Version     	: 0.8 beta2


#Version 0.8 beta2

#Changed: presets for lampshade and curtains to diffuse transmitter presets
#Added: support for multiple uv sets
#Added: dummy components and dummy instancing (proxies)
#Fixed: Sketchup view zooming in when setting render settings
#Changed: Now double-click a material in the paint bucket to edit it
#Added: Explode distorted texture tool
#Added: Support for export of nested blends
#Added: Support for loading and saving of IGM nested blends
#Added: Clip map channel to all materials


#Version 0.8 beta1

#Added: application and material observers to determine the current selected material in the Paint Bucket
#Changed: combined material browser with material editor
#Changed: render setting 'bidirectional' to 'tracing method' to allow selection of more methods including hybrid
#Added: support for diffuse transmitter material.  Change relative amount of reflectance and transmittance with the Sketchup opacity slider
#Added: Thin Glass material type

#Version 0.7.1 Official

#Added: new light presets
#Added: support for loading and saving IGM files (Blend materials not supported yet)
#Added: support for Indigo sphere primitive by naming component definition IndigoSphere (cannot be placed inside Indigo instance)
#Added: support for Indigo instancing
#Added: Sketchup Fog export
#Changed: Default Reinhard Pre-Scale to 6.0
#Added: Export selection only
#Removed: Camera is outside the sky-sphere warning message
#Changed: Indigo path is no longer saved in the Sketchup model, it is stored in a text file in the Sketchup/Plugins directory
#Fixed: command window staying open when auto-rendering
#Changed: Default image save period to 120 seconds
#Fixed: minor bugfixes


#version 0.7test7

#Changed: Re-enabled Bidirectional Path Tracing
#Changed: export file extension from XML to IGS
#Added: Option to save igi file in render settings
#Added: Advanced user functions accessible through Ruby Console
#Changed: Materials are now listed in alphabetical order in Material Editor


#Version 0.7test5

#Added: Environment Map Support
#Added: Up to 4 threads selectable
#Changed: Mirror preset is now a specular material with high IOR (perfect dielectric)
#Added: Support for most features of Indigo 0.7
#Changed: Removed tonemapping from render settings - use Indigo tonemapper GUI
#Added: Bump map support to metals (complex IOR)
#Changed: Changed default exponent to 1000
#Added: Material editor context menu
#Added: New sss material and light presets
#Changed: aperture setting is now specified as f-stop
#Changed: minor UI changes
#Added: autofocus is selectable through focus tool
#Added: reverse gamma correction to rgb meshlights
#Added: context menu to apply material presets and quick export
#Added: New Render Modes - Export All, Separate Settings, and Settings Only (Quick Export)
#Added: Option to run Indigo in Low Priority (creates batch file) - May only work with PCs
#Changed: Material Table to Query Material Types
#Changed: Only visible geometry is exported
#Changed: Auto-render only allowed when file is exported to Indigo directory
#Changed: BPT always turned off (not working in Indigo 0.7test5)
#Changed: Default MNCR to 500 from 100.  500 works better for SSS


#Version 0.6 Official

#Changes since Beta1 - Initial Test Release
#Fixed: [,,,,] characters corrupting inifile.txt on some sytems
#Changed: Auto-render now aborts if there are no lights in scene
#Changed: Changed the way NK_Data (Complex IOR) materials are selected
#Fixed: imported 3DS materials automatically being assigned Specular material type
#Fixed: applied reverse gamma correction exponent of 2.3 to diffuse color of materials
#Changed: Sketchup camera aspect ratio will change based on width and height of render.	
#Changed: Focal Length in Set Focus Tool changed to Focus Distance
#Added: Monochrome Render Modes
#Added: Dull plastic, shiny plastic, leather, car paint, complex IOR presets
#Changed: Specular Materials can now be transparent or non-transparent depending on the opacity setting
#Changed: Negative bump values are now allowed
#Changed: Changed default fresnel to 1.0
#-----------------------------------------------------------------------------

require 'sketchup.rb'
		

class SU2IN

# ---- Settings and global variables ----- #

def SU2IN::reset_global_variables

model = Sketchup.active_model

	$n_meshlights=0	
	$sun=false
	$n_cameras=0
	$face=0
	$g_su2in_scale = 0.0254
	$g_su2in_copy_textures = true
	$g_su2in_export_materials = true
	$g_su2in_export_meshes = true
	$g_su2in_export_lights = true
	$g_su2in_model_name="" if not $g_su2in_model_name
	$g_su2in_textures_prefix = "TX_"
	$g_su2in_model_textures=[]
	$g_su2in_count_tri = 0
	$g_su2in_count_material = 0
	$g_su2in_faces = []
	$g_su2in_trans = []
	$g_su2in_lights = []
	$animation=false
	$g_su2in_materials = {}
	$g_su2in_fm_materials = {}
	$g_su2in_selected=false	
	$g_su2in_clay=false
	$monochrome=false
	$parent_mat=[]
	$fm_comp=[]
	$current_camera=model.active_view.camera
	#$current_camera.image_width=35.0
	#$selected_material=nil if not $selected_material
	$default_material_exported=false
	$monochrome_material=""
	$null_exported=false
	$mode="All" if not $mode
	$mat_path="" if not $mat_path
	$ss_exported=false if not $ss_exported
	$settings_only=false if not $settings_only
	$definition_materials={}
	$component_definitions=[]
	$component_instances=[]
	$spheres=[]
	$exported_mats=[]

#Render Settings - Modify these values to change defaults

	
$width=(model.get_attribute 'indigo07_settings','w',"800").to_i
$height=(model.get_attribute 'indigo07_settings','h',"600").to_i
$aperture=(model.get_attribute 'indigo07_settings','ap',"f\/22").to_s
$white_balance=model.get_attribute 'indigo07_settings','wb',"E"
$environment=model.get_attribute 'indigo07_settings','env',"Sun and Sky"
$turbidity=(model.get_attribute 'indigo07_settings','tur',"2.0").to_f

#$bidir=model.get_attribute 'indigo07_settings','bidir',"Off"

$method=model.get_attribute 'indigo07_settings','method',"Hybrid"
$mncr=(model.get_attribute 'indigo07_settings','mncr',"500").to_i
$image_save_period=(model.get_attribute 'indigo07_settings','isp',"120").to_i
$halt_time=(model.get_attribute 'indigo07_settings','halt',"-1").to_i
$threads=(model.get_attribute 'indigo07_settings','threads',"1").to_s
$focal_point=(model.get_attribute 'indigo07_settings','fp',"auto")
$auto_render=model.get_attribute 'indigo07_settings','auto',"On"
$mode=model.get_attribute 'indigo07_settings','mode',"Separate Settings"
$save_igi=model.get_attribute 'indigo07_settings','save_igi',"On"
	
$exr=model.get_attribute 'indigo07_settings','exr',""
$exr_type=model.get_attribute 'indigo07_settings','exr_type',"Lat_Long"
$exr_width=(model.get_attribute 'indigo07_settings','exr_width',"640").to_i
$exr_gain=(model.get_attribute 'indigo07_settings','exr_gain',"1.0").to_f
	
 
end


##### ------  Exporting starts here ------ ######

def SU2IN::export

	
	SU2IN.reset_global_variables
	$mode="Settings Only" if $settings_only

	model = Sketchup.active_model
	
	if $mode!="Settings Only"

		$g_su2in_model_name=""
		model_filename = File.basename(model.path)
			if model_filename!=""
				model_name = model_filename.split(".")[0]
				model_name += ".igs"
			else
				model_name = "Untitled.igs" 	
			end
		
		$g_su2in_export_file=UI.savepanel("Export Model", "" , model_name)

		return if $g_su2in_export_file==nil		

			if $g_su2in_export_file==$g_su2in_export_file.split(".")[0] #no extension given
				$g_su2in_export_file+=".igs"
			end
		$g_su2in_model_name=File.basename($g_su2in_export_file)
		
		
		
		$mat_path=$g_su2in_export_file.split($g_su2in_model_name)[0] + $g_su2in_model_name.split(".")[0] + "-settings.igs"
		
		$g_su2in_model_name=$g_su2in_model_name.split(".")[0]
	end

	renderingoptions=model.rendering_options
	mode=renderingoptions["RenderMode"]
	
	
	
	if mode==5 and $mode!="Settings Only"
		materials = model.materials
    		translatednames = materials.collect {|m| m.display_name}
		names = materials.collect {|m| m.name}
		dropdowns=["Clay","Default"]
		dropdowns=dropdowns.push(translatednames)
		prompts=["Choose Material"]
		values=["Clay"]
		enums=[dropdowns.join("|")]
		results=inputbox prompts,values, enums,"Monochrome Render"
		return if not results
		if results[0]=="Clay"
			$g_su2in_clay=true
			$monochrome_material="Default"
		else
			$monochrome=true
			mat=results[0]
			$monochrome_material=mat.gsub(/[<>]/, '*')
					
		end
	end

	if mode==2
		
		#$xray=true
	end


	if $g_su2in_model_name!=""	
		
		$indigo_path=SU2IN.get_indigo_path

		if $indigo_path==nil
			UI.messagebox "indigo.exe not found.  Export aborted."
			return
		end
	
				
		out = File.new($g_su2in_export_file,"w") if $mode!="Settings Only"
		out_mat=File.new($mat_path,"w") if $mode!="All"

		$g_su2in_path_textures=File.dirname($g_su2in_export_file)

		start_time=Time.new

				

		if $mode=="All"

			out.puts "<?xml version=\"1.0\" standalone=no>"
			out.puts "<scene>"

			SU2IN.export_global_settings(out)

			SU2IN.param_cam_view(model.active_view, out)
		

			SU2IN.write_sky(out)

			materials = model.materials
			
			SU2IN.export_default_mat(out)
			for mat in materials
				SU2IN.save_mat(nil,mat,out) if not $exported_mats.index(mat)
				
			end

			SU2IN.export_mesh(out)
			SU2IN.export_spheres(out)
			SU2IN.export_fog(out) if model.rendering_options["DisplayFog"]

			out.puts "</scene>"
			out.close
			$ss_exported=false
			$mat_path=""
		else
			
			if $mode=="Separate Settings"
				out.puts "<?xml version=\"1.0\" standalone=no>"
				out.puts "<scene>"
				out.puts "		<include>"
				out.puts "		<pathname>#{File.basename($mat_path)}</pathname>"
				out.puts "		</include>"
				SU2IN.export_mesh(out)
				SU2IN.export_spheres(out)
				out.puts "</scene>"
				out.close
				
			end

			out_mat.puts "<scenedata>"

			SU2IN.export_global_settings(out_mat)
			SU2IN.param_cam_view(model.active_view, out_mat)
			SU2IN.write_sky(out_mat)

			materials = model.materials
			

			SU2IN.export_default_mat(out_mat)
			for mat in materials
				SU2IN.save_mat(nil,mat,out_mat) if not $exported_mats.index(mat)
				
			end
	

			SU2IN.export_fog(out_mat) if model.rendering_options["DisplayFog"]
			out_mat.puts "</scenedata>"
			out_mat.close
			$ss_exported=true
			
		end	
		

	if ($g_su2in_copy_textures == true)

		#su2in_texturewriter = Sketchup.create_texture_writer
		SU2IN.status_bar("Exporting textures")		

		if FileTest.exist? ($g_su2in_path_textures+"\\"+$g_su2in_textures_prefix+$g_su2in_model_name)
		else
			Dir.mkdir($g_su2in_path_textures+"\\"+$g_su2in_textures_prefix+$g_su2in_model_name)
		end

		status=SU2IN.export_textures
		
		if status
			stext="All textures and model"
		else
			stext="An error occured when exporting textures. Model"
		end		
		
		

	else
		stext = "Model"
	end

		
		end_time=Time.new
		elapsed=end_time-start_time
		time=" exported in "
			(time=time+"#{(elapsed/3600).floor}h ";elapsed-=(elapsed/3600).floor*3600) if (elapsed/3600).floor>0
			(time=time+"#{(elapsed/60).floor}m ";elapsed-=(elapsed/60).floor*60) if (elapsed/60).floor>0
			time=time+"#{elapsed.round}s. "

		SU2IN.status_bar(stext+time+" Triangles = #{$g_su2in_count_tri}")
		#UI.messagebox(stext+time+" Triangles = #{$g_su2in_count_tri}")
		export_text="Model saved in file:\n"
		
		(UI.messagebox export_text + $g_su2in_export_file) if $auto_render=="Off"

	end

	

		
#--------------Open Indigo for rendering
	if ($auto_render=="On") and (FileTest.exist?($indigo_path))
		if ($n_meshlights==0) and ($sun==false) and ($environment!="Background") and ($environment!="Environment Map") and ($mode!="Settings Only")
			UI.messagebox "Warning: There are no lights in your scene.  Auto-render aborted."
			UI.messagebox export_text + $g_su2in_export_file
			SU2IN.reset_global_variables
			return
		end
			
		command_line=create_batch_file($indigo_path)
		command_line=command_line.gsub("\\","\/")
		
		if File.dirname($indigo_path)==File.dirname($g_su2in_export_file)
			UI.openURL(command_line)
		else
			UI.messagebox ("To auto-render from Sketchup, you must export the model to the Indigo directory. Open \"indigo.exe\" to render.")
		end
		SU2IN.reset_global_variables
	else
		SU2IN.reset_global_variables
		
	end
	$settings_only=false
	
end


# ----------------------------------- Render Settings

def SU2IN::render_settings

SU2IN.reset_global_variables

#Get Current Settings

model=Sketchup.active_model

prompts=["Width","Height","Aperture (f-stop)","White Balance","Environment","Sky Turbidity (>0)","Tracing Method","Max Num Consec Rej (>0)","Image Save Period (s)","Halt Time (s)","Number of Threads","Auto-Render after Export","Export Mode","Save IGI"]
values=[$width,$height,$aperture,$white_balance,$environment,$turbidity,$method,$mncr,$image_save_period,$halt_time,$threads,$auto_render,$mode,$save_igi]
wb_choices=%w[E D50 D55 D65 D75 A B C 9300 F2 F7 F11].join("|")
env_choices=["Sun and Sky","Background","Environment Map"].join("|")
ap_choices=["f\/1.0","f\/1.4","f\/2","f\/2.8","f\/4","f\/5.6","f\/8","f\/11","f\/16","f\/22","f\/32","f\/45","f\/64","f\/90","f\/128"]
meth_choices=["MLT","PT","Hybrid","Bidir MLT","Bidir PT","Bidir Hybrid"].join("|")
th_choices=["Low Priority","1","2","3","4"].join("|")
auto_choices=%w[On Off].join("|")
mode_choices=["All","Separate Settings"].join("|")
save_igi_choices=%w[On Off].join("|")

dropdowns = [nil,nil,ap_choices.join("|"),wb_choices,env_choices,nil,meth_choices,nil,nil,nil,th_choices,auto_choices,mode_choices,save_igi_choices] 

results=inputbox prompts,values,dropdowns,"Indigo Render Settings"
return nil if not results

#DATA Validation

results[8]=-1 if results[8] <-1
model.start_operation "Render Settings"

#------------------------store settings in attribute dictionary and save to global variables

$width=model.set_attribute 'indigo07_settings','w',results[0].abs 
$height=model.set_attribute 'indigo07_settings','h',results[1].abs
$aperture=model.set_attribute 'indigo07_settings','ap',results[2]
$white_balance=model.set_attribute 'indigo07_settings','wb',results[3]
$environment=model.set_attribute 'indigo07_settings','env',results[4]
$turbidity=model.set_attribute 'indigo07_settings','tur',results[5].abs
$method=model.set_attribute 'indigo07_settings','method',results[6]
$mncr=model.set_attribute 'indigo07_settings','mncr',results[7].abs
$image_save_period=model.set_attribute 'indigo07_settings','isp',results[8].abs
$halt_time=model.set_attribute 'indigo07_settings','halt',results[9]
$threads=model.set_attribute 'indigo07_settings','threads',results[10]
$auto_render=model.set_attribute 'indigo07_settings','auto',results[11]
$mode=model.set_attribute 'indigo07_settings','mode',results[12]
$save_igi=model.set_attribute 'indigo07_settings','save_igi',results[13]

if $width==0
	result=UI.messagebox ("Click \"Yes\" to set height and width to viewport dimensions.\nClick \"No\" to reset to Sketchup default camera settings.", MB_YESNO)
		if result==6 #Yes
			$width=model.set_attribute 'indigo07_settings','w',Sketchup.active_model.active_view.vpwidth
			$height=model.set_attribute 'indigo07_settings','h',Sketchup.active_model.active_view.vpheight
		else
			$current_camera.aspect_ratio=0.0
			$current_camera.image_width=0.0
			return
		end
end

view_aspect=((model.active_view.vpwidth).to_f)/((model.active_view.vpheight).to_f)
old_aspect=$current_camera.aspect_ratio
old_aspect=1.0 if old_aspect==0.0
f=$current_camera.focal_length

#if old_aspect<view_aspect
	$current_camera.aspect_ratio=($width.to_f)/($height.to_f)
	$current_camera.focal_length=f/($current_camera.aspect_ratio/old_aspect) #if $current_camera.aspect_ratio<view_aspect
	#$current_camera.focal_length=f/($current_camera.aspect_ratio/old_aspect) if $current_camera.aspect_ratio>view_aspect
#else
	#$current_camera.focal_length=f/view_aspect if old_a
	#$current_camera.aspect_ratio=($width.to_f)/($height.to_f)
#end
#$current_camera.focal_length=f*($current_camera.aspect_ratio/old_aspect)

model.commit_operation
end

# ----------------------------------- About message
def SU2IN::about
UI.messagebox("SkIndigo 0.8 beta2 Released XX XXX 2007
Freeware SketchUp Exporter to Indigo 0.8 Stable
Author: Whaat\n\nBased on SU2KT written by Tomasz
SU2POV written by Didier Bur
and OGRE exporter by Kojack

Please see Indigo's website if you have questions about Indigo.
http://www.indigorenderer.com

HOW TO:

Render with Sun and Sky Light
1. Turn on shadows
2. Turn on Sun and Sky in SkIndigo Render Settings

Render with Custom Lights
1. Use either an RGB Light or a Blackbody Light material in your scene.
2. RGB lights will emit light according to the material color
3. Blackbody lights will emit light according to the temperature
4. Light is only emitted from the front side of the face

Render with Uniform Background Light
1. Turn on Background in SkIndigo Render Settings
2. Scene will be lit with a uniform background light matching the current Sketchup background color

Render with Environment Map
1. Turn on Environment Map in SkIndigo Render Settings
2. Set your Environment Map using the SkIndigo menu item
3. You can use EXR maps in Lat_Long format or FLOAT maps in Spherical format

Monochrome Render (Clay)
1. Change Sketchup render mode to monochrome
2. Choose Clay to render clay mode or any other material
3. Choose Default to render using the current default material color

Adjust the Rendered Image Tones and Exposure
1. Make sure you have downloaded the Indigo Tonemapper application.
2. Load the appropriate .igi file using the Indigo Tonemapper application.

Create Transparent Material
1. Enable transparency in Sketchup material settings using the opacity slider
2. Set the material type to SPECULAR
3. The opacity slider will change the amount of RGB absorptivity. Set to 0 for no absorptivity (i.e. clear glass).
4. SPECULAR materials should have thickness to render properly.

Create Material with Specular Highlights 
1. Set the material type to Phong
2. Set the index of refraction to >1.0 (High IOR=Very reflective)
3. Use the exponent value to control the glossiness. (High value=Smooth polished surface)

Create Diffuse (Lambertian) Material
1. Set the material type to Diffuse

Create Diffuse Transmitter Material
1. Set the material type to Diffuse
2. Use the Sketchup opacity slider to set the relative amount of transmittance and reflectance.

Use BLEND materials
1. Set the material type to BLEND
2. If a texture is applied to the material, it will be used as the clipping map
3. If no texture is applied, the blend factor will control the weight of the materials

Use Sub-surface scattering
1. Enable transparency in Sketchup material settings using the opacity slider
2. Set the material type to SPECULAR
3. The opacity slider will change the amount of RGB absorptivity. Set to 0 for no absorptivity
4. Enable SSS with the material editor
5. Input the Uniform scattering coefficient or the 3 RGB scattering coefficients separated by spaces
6. SPECULAR materials should have thickness to render properly.

Use Material Presets
1. Presets are intended to give you approximate starting values that you can further refine to suit your needs
2. These values will be updated in future releases as the Indigo material library continues to be developed

Use Textures
1. Apply textures directly to individual faces, not groups or components
2. Currently, textures distorted with the position texture tool are not supported
3. JPEG and PNG textures are supported but PNG textures with alpha channels may cause Indigo to crash

Use Bump and Exponent Maps
1. You can select any texture that is currently in your model
2. The map will be positioned using the same coordinates as the currently applied Sketchup texture.  If no Sketchup texture is applied, the bump map will not render correctly
3. If you wish to use a bump map different then the Sketchup texture,they should be the same size or you will get strange results.
4. Set bump or exponent gain

Use Fog Export
1. Enable Fog display within Sketchup
2. Set the left fog slider to 0
3. Set the right fog slider to the distance that and object will be hidden by fog
4. Fog increases render times substantially

Use Instancing
1. Right-click on any component to enable or disable instancing
2. Instances will all use the materials contained in the component definition
3. Components that contain Blackbody and RGB light materials cannot be instanced
4. Face-me components cannot be inside instances
5. Indigo Spheres cannot be inside instances

Use Indigo Sphere Primitives
1. Any component definition that is named IndigoSphere will be exported as an Indigo sphere primitive
2. The size of the sphere is based on the maximum dimension of the component's bounding box
3. An Indigo Sphere will use the material that is applied to the component instance.

Use UV Mapping (All actions performed via context menu)
1. Load all entities that have a particular material applied
2. Use the Sketchup position texture tool to position the texture or the Paint Bucket to change the dimensions of the texture
3. Save the current UV set (texture position) to an index
4. Switch all loaded entities to a different material (eg. bump map material)
5. Re-position the newly applied texture as before
6. Save the current UV set to a different index than step 3
7. Repeat steps 4-6 if desired (maximum of 4 UV sets)
8. Switch back to the original material
9. Assign the appropriate UV sets to the original material

Use Dummy Components
1. Create a new component that has the same size and the same axis location as the one you are wanting to substitue
2. Name the new component definition \"name_dummy\" where \"name\" is the name of the component that the dummy represents.
3. If you want to enable instancing for this component, make sure that you enable instancing for the dummy, not the original component.

Resume a Render from an .igi file
1. From the Ruby Console Window type \"Indigo.res\"
2. Locate the Indigo IGI file and then the Indigo IGS input file.

TIPS:

1. Use Sketchup shortcut keys!  Especially for selecting the material editor
2. Be sure that no faces are reversed.  Use monochrome mode to check for reversed faces
3. Use layers to separate geometry and lights.  For example, put different lighting set-ups on different layers.  If you are doing material tests, put unnecessary objects on a hidden layer
4. Always export your model to the Indigo directory.  This allows use of Auto-render
5. If you are getting strange shadows in your render, try disabling smoothing.  Also, check orientation of faces
6. Modify the aperture size to control depth of field
7. If your image is blurry, use the Set Camera Focus Tool

Indigo 0.8 features currently not supported:
1. Peak spectra
2. Henyey Greenstein phase function
3. Dermis and epidermis media
4. a and c texture coefficients
5. Comprehensive nkdata list

KNOWN ISSUES/BUGS:
1. Indigo sphere primitives cannot be inside instances
2. Meshlights cannot be inside instances 
3. Face-me components cannot be inside instances
3. Distorted textures not yet supported
4. Textures applied to groups and components not yet supported

" , MB_MULTILINE , "About SkIndigo 0.8 beta2 - Exporter to Indigo 0.8") 

end

# -----------Extract the camera parameters of the current view ------------------------------------
def SU2IN::param_cam_view( v , out)

h = v.vpheight
w = v.vpwidth
user_camera = v.camera
user_pers = user_camera.perspective?
user_eye = user_camera.eye

user_x = user_camera.xaxis 
user_y = user_camera.yaxis 
user_z = user_camera.zaxis


	if( user_pers == true ) 
  	user_foc = user_camera.focal_length

	else 
		user_foc = 36

	end
	
	w_x_h = w.to_s + "x" + h.to_s
	w_x_h=$resolution if $animation==true
	SU2IN.write_camera("\#\# Current View \#\#", user_eye, user_x, user_y, user_z, w_x_h, user_foc, out)

end


#---------------- Save camera to file ------------------------#

def SU2IN::write_camera( p, user_eye, user_x, user_y, user_z, w_x_h, user_foc, out)


cx=user_eye.x.to_m
cy=user_eye.y.to_m
cz=user_eye.z.to_m


rvx=user_x.x.to_cm
rvy=user_x.y.to_cm
rvz=user_x.z.to_cm

uvx=user_y.x.to_cm
uvy=user_y.y.to_cm
uvz=user_y.z.to_cm

fvx=user_z.x.to_cm
fvy=user_z.y.to_cm
fvz=user_z.z.to_cm

#image_width=$current_camera.image_width

f=$current_camera.focal_length
sensor_width=$current_camera.image_width.to_f/1000
sensor_width=0.036 if sensor_width==0.0
old_aspect=$current_camera.aspect_ratio

#$current_camera.image_width=35.0

if old_aspect==0.0
	view=Sketchup.active_model.active_view
	old_aspect=view.vpwidth.to_f/view.vpheight.to_f
	sensor_distance=(f.to_f/old_aspect)/1000
else
	sensor_distance=f.to_f/1000
end

#aspect=$width.to_f/$height.to_f
aspect=old_aspect


#$current_camera.image_width=image_width

stop=$aperture.split("\/")[1].to_f
ap_radius=sensor_distance/(stop*2)

if $focal_point!="auto"
	vector=user_eye - $focal_point if $focal_point
	$focus_distance=vector.length.to_m
else
	$focus_distance=2.0
end


out.puts "		<camera>"
out.puts "			<pos>" + "%.4f" %(cx)+ " "+ "%.4f" % (cy) + " " + "%.4f" %(cz)+ "</pos>"
out.puts "			<up>" + "%.4f" %(uvx)+ " "+ "%.4f" % (uvy) + " " + "%.4f" %(uvz)+ "</up>"
out.puts "			<forwards>" + "%.4f" %(fvx)+ " "+ "%.4f" % (fvy) + " " + "%.4f" %(fvz)+ "</forwards>"
out.puts "			<aperture_radius>" + "%.4f" %(ap_radius)+"</aperture_radius>"
out.puts "			<focus_distance>"+ "%.4f" %($focus_distance)+"</focus_distance>"
out.puts "			<aspect_ratio>" + "%.4f" %(aspect) + "</aspect_ratio>"
out.puts "			<sensor_width>#{sensor_width}</sensor_width>"
out.puts "			<lens_sensor_dist>" + "%.4f" %(sensor_distance)+"</lens_sensor_dist>"
out.puts "			<white_balance>#{$white_balance}</white_balance>"
out.puts "			<autofocus/>" if $focal_point=="auto"

out.puts "		</camera>"



end

#method to get an objects color

def SU2IN::get_object_color (obj)
  if (obj.material == nil)
    col = Sketchup::Color.new [255,255,255]
    else
    col = obj.material.color
  end
return col
end


# -------------------------------- Export Physical Sky and Background settings

def SU2IN::write_sky ( out )
	si = Sketchup.active_model.shadow_info
	
	if si["DisplayShadows"]==true	#DisplayShadows True or False
		$sun=true
	else
		$sun=false
	end
	
	
	v = si["SunDirection"]

	point = Geom::Point3d.new 0,0,0
	xvector = Geom::Vector3d.new 1,0,0
	yvector = Geom::Vector3d.new 0,1,0
	zvector = Geom::Vector3d.new 0,0,1
	nangle = si["NorthAngle"].degrees
	tr = Geom::Transformation.rotation point, zvector, -nangle
	tr2 = Geom::Transformation.rotation point, zvector, nangle
	ptx=xvector.transform! tr     
	pty=yvector.transform! tr     
	v=v.transform! tr2		#Sun direction has to be rotated in opposite direction..
	shadow = Sketchup.active_model.shadow_info.to_a

	if ($sun==true) and ($environment=="Sun and Sky")

	
	
		out.puts "		<skylight>"
		out.puts "			<sundir>"+ "%.4f" % (v.x) + " " + "%.4f" % (v.y) + " " + "%.4f" % (v.z) + "</sundir>"
		out.puts "			<turbidity>#{$turbidity}</turbidity>"
		out.puts "		</skylight>"


	else
		if $environment=="Background"

			result=Sketchup.active_model.rendering_options["BackgroundColor"]
			
			if result
				red=result.red*200000
				green=result.green*200000
				blue=result.blue*200000
			else
				red=0.0
				green=0.0
				blue=0.0
			end
			
			out.puts "		<background>"
			out.puts "			<spectrum>"
			out.puts "				<rgb>"
			out.print "					<rgb>#{red} #{green} #{blue}</rgb>\n"
			out.puts "				</rgb>"
			out.puts "			</spectrum>"
			out.puts "		</background>"
		end
		
		if $environment=="Environment Map"
			UI.messagebox "Warning:No Environment Map has been set." if $exr==""
			out.puts "		<env_map>"
			if $exr_type=="Lat_Long"
				out.puts "			<latlong>"
				#out.puts "				<path>probes\\#{$exr}</path>"
				out.puts "				<path>#{$exr}</path>"
				out.puts "				<gain>#{$exr_gain}</gain>"
				out.puts "			</latlong>"
				out.puts "		</env_map>"
			else
				if $exr_type=="Spherical"
				out.puts "			<spherical>"
				#out.puts "				<path>probes\\#{$exr}</path>"
				out.puts "				<path>#{$exr}</path>"
				out.puts "				<width>#{$exr_width}</width>"
				out.puts "				<gain>#{$exr_gain}</gain>"
				out.puts "			</spherical>"
				out.puts "		</env_map>"
				end
			end
		end
	end

end


#### -------------- Export rendering settings ----------------- ##########

def SU2IN::export_global_settings(out)

case $method
	when "MLT"
		met=true
		bidir=false
		hybrid=false
	when "PT"
		met=false
		bidir=false
		hybrid=false
	when "Hybrid"
		met=false
		bidir=false
		hybrid=true
	when "Bidir MLT"
		met=true
		bidir=true
		hybrid=false
	when "Bidir PT"
		met=false
		bidir=true
		hybrid=false
	when "Bidir Hybrid"
		met=false
		bidir=true
		hybrid=true

end

old_aspect=$current_camera.aspect_ratio
if old_aspect==0.0
	view=Sketchup.active_model.active_view
	old_aspect=view.vpwidth.to_f/view.vpheight.to_f
end
aspect=$width.to_f/$height.to_f
if old_aspect!=aspect #this is possible if using photomatch
	$height=($width.to_f/old_aspect).to_i #set new image height so that width to height ratio will match aspect ratio of the photomatch
end
	

out.puts "		<renderer_settings>"
out.puts "			<width>#{$width}</width>"
out.puts "			<height>#{$height}</height>"
out.puts "			<max_num_consec_rejections>#{$mncr}</max_num_consec_rejections>"
out.puts "			<bidirectional>true</bidirectional>" if bidir
out.puts "			<bidirectional>false</bidirectional>" if not bidir
out.puts "			<metropolis>true</metropolis>" if met
out.puts "			<metropolis>false</metropolis>" if not met
out.puts "			<hybrid>true</hybrid>" if hybrid
out.puts "			<hybrid>false</hybrid>" if not hybrid
out.puts "			<halt_time>#{$halt_time}</halt_time>"
out.puts "			<image_save_period>#{$image_save_period}</image_save_period>"
out.puts "			<num_threads>#{$threads}</num_threads>" if $threads !="Low Priority"
out.puts "			<save_igi>true</save_igi>" if $save_igi=="On"
out.puts "			<save_igi>false</save_igi>" if $save_igi=="Off"
out.puts "		</renderer_settings>"


out.puts "		<tonemapping>"
out.puts "			<reinhard>"
out.puts "				<pre_scale>6.0</pre_scale>"
out.puts "				<post_scale>1.0</post_scale>"
out.puts "				<burn>6.0</burn>"
out.puts "			</reinhard>"
out.puts "		</tonemapping>"



end

##### --- EXPORT DEFAULT MATERIAL --- #####
def SU2IN::export_default_mat(out)

colour = Sketchup.active_model.rendering_options["FaceFrontColor"]
result=colour.red,colour.green,colour.blue
result=[216,216,216] if $g_su2in_clay==true
out.puts "		<material>"
out.print "			<name>Default</name>\n"
out.print "			<diffuse>\n"
out.puts  "				<colour>" + "%.4f" %((result[0]/255.0)) + " "+ "%.4f" % ((result[1]/255.0)) + " "+ "%.4f" % ((result[2]/255.0)) + "</colour>"
out.puts "			</diffuse>"
#out.puts  "				<colour>" + "%.4f" %((rand)) + " "+ "%.4f" % ((rand)) + " "+ "%.4f" % ((rand)) + "</colour>"#out.print " 			</diffuse>\n"
#out.print "			<phong>\n"
#out.puts  "				<diffuse>" + "%.4f" %((rand)) + " "+ "%.4f" % ((rand)) + " "+ "%.4f" % ((rand)) + "</diffuse>"
#out.puts "				<ior>1.72</ior>\n"
#out.print " 				<fresnel_scale>1.0</fresnel_scale>\n"
#out.print " 				<exponent>30000</exponent>\n"
#out.print " 			</phong>\n"
out.print " 		</material>\n"
$default_material_exported=true

end

##### --- EXPORT MATERIAL -----#####
def SU2IN::export_mat(out,mat,all)

if mat !=nil
	mat_name=mat.display_name
	mat_name=mat_name.gsub(/[<>]/, '*') 
end

##################xray material test####################
if $xray
	if not $null_exported
			out.puts "		<material>"
			out.puts "			<name>None</name>"
			out.puts "			<null_material/>"
			out.puts "		</material>"
			$null_exported=true
	end
	xray_name=mat_name
	mat_name="_xray_"+mat_name+"_xray_"
end


mat.attribute_dictionary 'indigo07_materials' if mat
if mat
	
	if mat.alpha<1.0
		type=mat.get_attribute 'indigo07_materials','type',"Specular"
	else
		type=mat.get_attribute 'indigo07_materials','type',"Diffuse"
	end
end


	
	#export specular material 

	if (mat != nil and type=="Specular")
		bump=mat.get_attribute 'indigo07_materials','bump_map',"None"
		result=mat.color.red,mat.color.green,mat.color.blue
		
	
		alpha_factor=mat.alpha*mat.alpha*1000

		sss_spectrum=[]
		sss_spectrum=(mat.get_attribute 'indigo07_materials','sssval',"1000").to_s.split(' ')
		if sss_spectrum[0] and sss_spectrum[1] and sss_spectrum[2]
			rgb_spectrum=true
		else
			rgb_spectrum=false
		end
		
		#Export the medium
		medium_name=mat_name+"_medium"
		out.puts "		<medium>"
		out.print "			<name>#{medium_name}</name>\n"
		out.print "			<precedence>#{mat.get_attribute 'indigo07_materials','precedence',10}</precedence>\n"
		out.print "			<basic>\n"
		out.print "				<ior>#{mat.get_attribute 'indigo07_materials','ior',1.52}</ior>\n"
		out.print "				<cauchy_b_coeff>#{mat.get_attribute 'indigo07_materials','cau',0.0}</cauchy_b_coeff>\n"
		out.puts "				<absorption_coefficient_spectrum>"
		out.puts "					<rgb>"
		out.print "						<rgb>" + "%.4f" %((1.0-result[0]/255.0)*alpha_factor) + " "+ "%.4f" %((1.0-result[1]/255.0)*alpha_factor) + " "+ "%.4f" %((1.0-result[2]/255.0)*alpha_factor) + "</rgb>\n"
		out.print "					</rgb>\n"
		out.print "				</absorption_coefficient_spectrum>\n"
		if (mat.get_attribute 'indigo07_materials','sss',"No")=="Yes"
			out.print "				<subsurface_scattering>\n"
			out.print "					<scattering_coefficient_spectrum>\n"
			
			if rgb_spectrum==true
				out.print "						<rgb>\n"
				out.print "							<rgb>#{sss_spectrum[0]} #{sss_spectrum[1]} #{sss_spectrum[2]}</rgb>\n"
				out.print "						</rgb>\n"
			else
				out.print "						<uniform>\n"
				out.print "							<value>#{sss_spectrum[0]}</value>\n"
				out.print "						</uniform>\n"
			end

			out.print "					</scattering_coefficient_spectrum>\n"
			out.print "					<phase_function>\n"
			out.print "						<uniform/>\n"
			out.print "					</phase_function>\n"
			out.print "				</subsurface_scattering>\n"
			
		end
		out.print "			</basic>\n"
		out.print "		</medium>\n"
		
		#Export the material
	
		clip_map=mat.get_attribute 'indigo07_materials','clip_map',"None"
		mat_name="C_"+mat_name+"_C" if clip_map!="None"

		out.print "		<material>\n"
		out.print " 		<name>#{mat_name}</name>\n"
		out.print " 		<specular>\n"
		out.print " 			<transparent>true</transparent>\n" if mat.alpha<1.0
		out.print "			<transparent>false</transparent>\n" if mat.alpha==1.0
		out.print "			<internal_medium_name>#{medium_name}</internal_medium_name>\n"

		SU2IN.export_bump_map(out,mat,bump,all) if bump!="None"
		
		out.print " 		</specular>\n"
		out.print " 		</material>\n"

		SU2IN.export_clip_map(out,mat,mat_name,clip_map,all) if clip_map!="None"
		
	end

	#export glossy transparent material

	if (mat != nil and type=="Glossy Transparent")
		bump=mat.get_attribute 'indigo07_materials','bump_map',"None"
		exp_map=mat.get_attribute 'indigo07_materials','exp_map',"None"
		result=mat.color.red,mat.color.green,mat.color.blue
		alpha_factor=mat.alpha*mat.alpha*1000
		
		#Export the medium
		medium_name=mat_name+"_medium"
		out.puts "		<medium>"
		out.print "			<name>#{medium_name}</name>\n"
		out.print "			<precedence>#{mat.get_attribute 'indigo07_materials','precedence',10}</precedence>\n"
		out.print "			<basic>\n"
		out.print "				<ior>#{mat.get_attribute 'indigo07_materials','ior',1.52}</ior>\n"
		out.print "				<cauchy_b_coeff>#{mat.get_attribute 'indigo07_materials','cau',0.0}</cauchy_b_coeff>\n"
		out.puts "				<absorption_coefficient_spectrum>"
		out.puts "					<rgb>"
		out.print "						<rgb>" + "%.4f" %((1.0-result[0]/255.0)*alpha_factor) + " "+ "%.4f" %((1.0-result[1]/255.0)*alpha_factor) + " "+ "%.4f" %((1.0-result[2]/255.0)*alpha_factor) + "</rgb>\n"
		out.print "					</rgb>\n"
		out.print "				</absorption_coefficient_spectrum>\n"
		if (mat.get_attribute 'indigo07_materials','sss',"No")=="Yes"
			out.print "				<subsurface_scattering>\n"
			out.print "					<scattering_coefficient_spectrum>\n"
			out.print "						<uniform>\n"
			out.print "							<value>#{mat.get_attribute 'indigo07_materials','sssval',"1000"}</value>\n"
			out.print "						</uniform>\n"
			out.print "					</scattering_coefficient_spectrum>\n"
			out.print "					<phase_function>\n"
			out.print "						<uniform/>\n"
			out.print "					</phase_function>\n"
			out.print "				</subsurface_scattering>\n"
			
		end
		out.print "			</basic>\n"
		out.print "		</medium>\n"
		
		#Export the material

		clip_map=mat.get_attribute 'indigo07_materials','clip_map',"None"
		mat_name="C_"+mat_name+"_C" if clip_map!="None"

		out.print "		<material>\n"
		out.print " 		<name>#{mat_name}</name>\n"
		out.print " 		<glossy_transparent>\n"
		out.print "			<internal_medium_name>#{medium_name}</internal_medium_name>\n"
		out.print "			<exponent>#{mat.get_attribute 'indigo07_materials','exp',1000}</exponent>\n"

		SU2IN.export_bump_map(out,mat,bump,all) if bump!="None"
		SU2IN.export_exp_map(out,mat,exp_map,all) if exp_map!="None"
		
		out.print " 		</glossy_transparent>\n"
		out.print " 		</material>\n"

		SU2IN.export_clip_map(out,mat,mat_name,clip_map,all) if clip_map!="None"
	end

#export diffuse material and diffuse transmitter
	
	if (mat !=nil and type=="Diffuse" )
					
		result=mat.color.red,mat.color.green,mat.color.blue
		bump=mat.get_attribute 'indigo07_materials','bump_map',"None"
		clip_map=mat.get_attribute 'indigo07_materials','clip_map',"None"
		mat_name="C_"+mat_name+"_C" if clip_map!="None"

		opacity=mat.alpha
		if opacity<1.0
			mat_a="DT_"+mat_name+"_DT"
			mat_b="D_"+mat_name+"_D"
		end

		out.puts "		<material>"
		out.print "			<name>#{mat_name}</name>\n" if opacity==1.0
		out.print "			<name>#{mat_b}</name>\n" if opacity<1.0
		out.print "			<diffuse>\n" 
		
		if (mat.texture == nil)
			out.puts  "				<colour>" + "%.4f" %((result[0]/255.0)) + " "+ "%.4f" % ((result[1]/255.0)) + " "+ "%.4f" % ((result[2]/255.0)) + "</colour>"
		else
			out.print "				<colour>1.0 1.0 1.0</colour>\n"
			SU2IN.export_albedo_map(out,mat,all)
		end
			
		SU2IN.export_bump_map(out,mat,bump,all) if bump!="None"

		out.print "			</diffuse>\n" 
		out.print "		</material>\n"

		if opacity<1.0    #We have a diffuse transmitter material
			
			out.puts "		<material>"
			out.print "			<name>#{mat_a}</name>\n"
			out.print "			<diffuse_transmitter>\n"
		
			if (mat.texture == nil)
				out.puts  "				<colour>" + "%.4f" %((result[0]/255.0)) + " "+ "%.4f" % ((result[1]/255.0)) + " "+ "%.4f" % ((result[2]/255.0)) + "</colour>"
			else
				out.print "				<colour>1.0 1.0 1.0</colour>\n"
				SU2IN.export_albedo_map(out,mat,all)
			end
			
			SU2IN.export_bump_map(out,mat,bump,all) if bump!="None"
			
			out.print "			</diffuse_transmitter>\n"
			out.print "		</material>\n"

			##########Export the blended diffuse transmitter

			out.puts "		<material>"
			out.print "			<name>#{mat_name}</name>\n"
			out.print "			<blend>\n"
			out.print "				<a_name>#{mat_a}</a_name>\n" #transmitter component
			out.print "				<b_name>#{mat_b}</b_name>\n" #normal diffuse component 	
			out.print "				<blend_factor>#{opacity}</blend_factor>\n"
			out.print "			</blend>\n"
			out.print "		</material>\n"

		end #if opacity less than 1.0
		SU2IN.export_clip_map(out,mat,mat_name,clip_map,all) if clip_map!="None"
		

	end #if type is diffuse


	#export phong material

	if (mat !=nil and type== "Phong" )
		result=mat.color.red,mat.color.green,mat.color.blue
		
		bump=mat.get_attribute 'indigo07_materials','bump_map',"None"
		exp_map=mat.get_attribute 'indigo07_materials','exp_map',"None"
		clip_map=mat.get_attribute 'indigo07_materials','clip_map',"None"
		mat_name="C_"+mat_name+"_C" if clip_map!="None"

		out.puts "		<material>"
		out.print "			<name>#{mat_name}</name>\n"
		out.print "			<phong>\n"
		if (mat.texture == nil)
			out.puts  "				<diffuse>" + "%.4f" %((result[0]/255.0)) + " "+ "%.4f" % ((result[1]/255.0)) + " "+ "%.4f" % ((result[2]/255.0)) + "</diffuse>"
		else
			out.puts  "				<diffuse>1.0 1.0 1.0</diffuse>"
		end
		out.puts "				<ior>#{mat.get_attribute 'indigo07_materials','ior',1.52}</ior>\n"
		out.print " 				<fresnel_scale>#{mat.get_attribute 'indigo07_materials','fres',1.0}</fresnel_scale>\n"
		out.print " 				<exponent>#{mat.get_attribute 'indigo07_materials', 'exp' ,1000}</exponent>\n"
		
		SU2IN.export_albedo_map(out,mat,all) if mat.texture!=nil
		SU2IN.export_bump_map(out,mat,bump,all) if bump!="None"
		SU2IN.export_exp_map(out,mat,exp_map,all) if exp_map!="None"

		out.print " 			</phong>\n"
		out.print " 		</material>\n"	

		SU2IN.export_clip_map(out,mat,mat_name,clip_map,all) if clip_map!="None"
	end
	
	#export NK_data materials
	
	if (mat !=nil)
		if (type=="Al") or (type=="Cr") or (type=="Cu") or (type=="Au") or (type=="Ag") or (type=="Ti") or (type=="W") or (type=="Li") or (type=="Ni") or (type=="Pt") 
			exponent=mat.get_attribute 'indigo07_materials','exp',"1000"
			bump=mat.get_attribute 'indigo07_materials','bump_map',"None"
			exp_map=mat.get_attribute 'indigo07_materials','exp_map',"None"
			clip_map=mat.get_attribute 'indigo07_materials','clip_map',"None"
			mat_name="C_"+mat_name+"_C" if clip_map!="None"
		
			out.puts "		<material>"
			out.puts "		<name>#{mat_name}</name>"
			out.puts "			<phong>"
			out.puts "				<nk_data>nkdata/#{type}.nk</nk_data>"
			out.puts "				<exponent>#{exponent}</exponent>"
			
			SU2IN.export_bump_map(out,mat,bump,all) if bump!="None"
			SU2IN.export_exp_map(out,mat,exp_map,all) if exp_map!="None"		
		
			out.puts "			</phong>"
			out.puts "		</material>"
			SU2IN.export_clip_map(out,mat,mat_name,clip_map,all) if clip_map!="None"
		end
	end
	
	#export blend material

	if (mat !=nil and type=="Blend")
				
		
		clip_map=mat.get_attribute 'indigo07_materials','clip_map',"None"
		mat_name="C_"+mat_name+"_C" if clip_map!="None"

		matA=mat.get_attribute 'indigo_blend','matA'
		matB=mat.get_attribute 'indigo_blend','matB'
		materials = Sketchup.active_model.materials
   		translatednames = materials.collect {|m| m.display_name}
		indexA = translatednames.index(matA)
		indexB = translatednames.index(matB)
		if (indexA==nil and matA!="None")
			UI.messagebox ("Error: Blend material tried to use non-existent material named") + " #{matA}"
		end
		if (indexB==nil and matB!="None")
			UI.messagebox ("Error: Blend material tried to use non-existent material named") + " #{matB}"
		end
		matA=matA.gsub(/[<>]/, '*')
		matB=matB.gsub(/[<>]/, '*')
		if ((matA=="None") or (matB=="None")) and $null_exported==false
			out.puts "		<material>"
			out.puts "			<name>None</name>"
			out.puts "			<null_material/>"
			out.puts "		</material>"
			$null_exported=true
		end
				
		out.puts "		<material>"
		out.print "			<name>#{mat_name}</name>\n"
		out.print "			<blend>\n"
		out.print "				<a_name>#{matA}</a_name>\n"
		out.print "				<b_name>#{matB}</b_name>\n"
		out.print "				<blend_factor>#{mat.get_attribute 'indigo_blend','blendfactor'}</blend_factor>\n"
		
		SU2IN.export_blend_map(out,mat,all) if mat.texture!=nil
				
		out.print "			</blend>\n"
		out.print "		</material>\n"

		SU2IN.export_clip_map(out,mat,mat_name,clip_map,all) if clip_map!="None"
	end

#export Thin Glass material
	
	if (mat !=nil and type=="Thin Glass" )
		opacity=mat.alpha
		mat_a="PH_"+mat_name+"_PH"
		mat_b="NULL_"+mat_name+"_NULL"
		result=mat.color.red,mat.color.green,mat.color.blue
		bump=mat.get_attribute 'indigo07_materials','bump_map',"None"
		exp_map=mat.get_attribute 'indigo07_materials','exp_map',"None"
		disable_texture=mat.get_attribute 'indigo07_materials','dis_tex',"No"
			
		out.puts "		<material>"
		out.print "			<name>#{mat_a}</name>\n"
		out.print "			<phong>\n"
		out.puts  "				<diffuse>" + "%.4f" %((result[0]/255.0)) + " "+ "%.4f" % ((result[1]/255.0)) + " "+ "%.4f" % ((result[2]/255.0)) + "</diffuse>"
		out.puts "				<ior>1000</ior>\n"
		out.print " 				<fresnel_scale>1.0</fresnel_scale>\n"
		out.print " 				<exponent>#{mat.get_attribute 'indigo07_materials', 'exp' ,1000}</exponent>\n"
						
		SU2IN.export_bump_map(out,mat,bump,all) if bump!="None"
		SU2IN.export_exp_map(out,mat,exp_map,all) if exp_map!="None"

		out.print " 			</phong>\n"
		out.print " 		</material>\n"
		################################Export Transmittance Layer###################
		out.puts "		<material>"
		out.print "			<name>#{mat_b}</name>\n"
		out.print "			<null_material/>\n"
		out.print "		</material>\n"
		################################Export Blended Material######################
		out.puts "		<material>"
		out.print "			<name>#{mat_name}</name>\n"
		out.print "			<blend>\n"
		out.print "				<a_name>#{mat_b}</a_name>\n" 
		out.print "				<b_name>#{mat_a}</b_name>\n" 	
		out.print "				<blend_factor>#{opacity}</blend_factor>\n"
		out.print "			</blend>\n"
		out.print "		</material>\n"
		#return
	end

######################xray test#############################
	if $xray and (type!="RGB Light") and (type!="Blackbody Light")
		out.puts "		<material>"
		out.print "			<name>#{xray_name}</name>\n"
		out.print "			<blend>\n"
		out.print "				<a_name>None</a_name>\n" 
		out.print "				<b_name>#{mat_name}</b_name>\n"
		out.print "				<blend_factor>0.3</blend_factor>\n"
		out.print "			</blend>\n"
		out.print "		</material>\n"
	end

end

##### --- EXPORT ALBEDO MAP -----######

def SU2IN::export_albedo_map(out,mat,all)

	uv_index=(mat.get_attribute 'uvsets','albedo',"Default")
	uv_index="0" if uv_index=="Default"
		
	out.print " 				<albedo_texture>\n"
	out.print "					<uv_set>uv#{uv_index}</uv_set>\n"
	out.print "					<path>#{SU2IN.trunc_path(mat.texture.filename)}</path>\n" if all
	out.print "					<path>#{File.basename(mat.texture.filename)}</path>\n" if not all
	out.print " 					<exponent>2.2</exponent>\n"
	out.print " 				</albedo_texture>\n"
	
end 


##### --- EXPORT BUMP MAP -----######

def SU2IN::export_bump_map(out,mat,bm,all)
 	
	uv_index=(mat.get_attribute 'uvsets','bump',"Default")
	uv_index="0" if uv_index=="Default"

	model = Sketchup.active_model
 	materials = model.materials
  	translatednames = materials.collect {|m| m.display_name}
	names=materials.collect {|m| m.name}
	bump_index=translatednames.index(bm)
	bump_material=bump_index ? materials[names[bump_index]] : nil
		if (not bump_material)
			UI.messagebox ("Could not find bump map for material named") + " #{mat.display_name}"
			return
		end
	if bump_material.texture!=nil
		out.print "				<bump_map>\n"
		out.print "					<uv_set>uv#{uv_index}</uv_set>\n"
		out.print "					<path>#{SU2IN.trunc_path(bump_material.texture.filename)}</path>\n" if all
		out.print "					<path>#{File.basename(bump_material.texture.filename)}</path>\n" if not all	
		out.print "					<b>#{mat.get_attribute 'indigo07_materials','bump_value'}</b>\n"
		out.print "					<exponent>1.0</exponent>\n"
		out.print " 				</bump_map>\n"
	else
		UI.messagebox ("Could not find bump map for material named") + " #{mat.display_name}"
	end
end 

##### --- EXPORT EXPONENT MAP -----######

def SU2IN::export_exp_map(out,mat,em,all)
 	
	uv_index=(mat.get_attribute 'uvsets','exponent',"Default")
	uv_index="0" if uv_index=="Default"

	model = Sketchup.active_model
 	materials = model.materials
  	translatednames = materials.collect {|m| m.display_name}
	names=materials.collect {|m| m.name}
	exp_index=translatednames.index(em)
	exp_material=exp_index ? materials[names[exp_index]] : nil
		if (not exp_material)
			UI.messagebox ("Could not find exponent map for material named") + " #{mat.display_name}"
			return
		end
	if exp_material.texture!=nil
		out.print "				<exponent_map>\n"
		out.print "					<uv_set>uv#{uv_index}</uv_set>\n"
		out.print "					<path>#{SU2IN.trunc_path(exp_material.texture.filename)}</path>\n" if all
		out.print "					<path>#{File.basename(exp_material.texture.filename)}</path>\n" if not all	
		out.print "					<b>#{mat.get_attribute 'indigo07_materials','expmap_value'}</b>\n"
		out.print "					<exponent>1.0</exponent>\n"
		out.print " 				</exponent_map>\n"
	else
		UI.messagebox ("Could not find exponent map for material named") + " #{mat.display_name}"
	end
end 

#### --- EXPORT BLEND MAP -----######

def SU2IN::export_blend_map(out,mat,all)

	uv_index=(mat.get_attribute 'uvsets','blend',"Default")
	uv_index="0" if uv_index=="Default"

	out.print "				<blend_map>\n"
	out.print "					<uv_set>uv#{uv_index}</uv_set>\n"
	out.print "					<path>#{SU2IN.trunc_path(mat.texture.filename)}</path>\n" if all
	out.print "					<path>#{File.basename(mat.texture.filename)}</path>\n" if not all
	out.print " 					<exponent>1.0</exponent>\n"
	out.print " 				</blend_map>\n"
	
end

#### --- EXPORT CLIP MAP (Blended Material) -----######

def SU2IN::export_clip_map(out,mat,clipped_name,cm,all)

	uv_index=(mat.get_attribute 'uvsets','clip',"Default")
	uv_index="0" if uv_index=="Default"
	
	mat_name=mat.display_name
	mat_name=mat_name.gsub(/[<>]/, '*') 

	model = Sketchup.active_model
 	materials = model.materials
  	translatednames = materials.collect {|m| m.display_name}
	names=materials.collect {|m| m.name}
	clip_index=translatednames.index(cm)
	clip_material=clip_index ? materials[names[clip_index]] : nil
		if (not clip_material)
			UI.messagebox ("Could not find clip map for material named") + " #{mat.display_name}"
			return
		end
	if clip_material.texture!=nil
		if not $null_exported
			out.puts "		<material>"
			out.puts "			<name>None</name>"
			out.puts "			<null_material/>"
			out.puts "		</material>"
			$null_exported=true
		end
	out.puts "		<material>"
	out.print "			<name>#{mat_name}</name>\n"
	out.print "			<blend>\n"
	out.print "				<a_name>#{clipped_name}</a_name>\n"
	out.print "				<b_name>None</b_name>\n"
	out.print "				<blend_factor>0.5</blend_factor>\n"
	out.print "				<blend_map>\n"
	out.print "					<uv_set>uv#{uv_index}</uv_set>\n"
	out.print "					<path>#{SU2IN.trunc_path(clip_material.texture.filename)}</path>\n" if all
	out.print "					<path>#{File.basename(clip_material.texture.filename)}</path>\n" if not all
	out.print " 					<exponent>1.0</exponent>\n"
	out.print " 				</blend_map>\n"	
	out.print "			</blend>\n"
	out.print "		</material>\n"
	else
		UI.messagebox ("Could not find clip map for material named") + " #{mat.display_name}"
	end
end


##### --- EXPORT CLAY MATERIAL -----#####

def SU2IN::export_clay(out)
	
	out.puts "		<material>"
	out.print "			<name>Clay</name>"
	out.print "			<diffuse>"
	out.print "				<colour>0.85 0.85 0.76</colour>" 
	out.print "			</diffuse>"
	out.print " 	</material>"

end


#### - Send text to status bar - ####
def SU2IN::status_bar(stat_text)
	
	statbar = Sketchup.set_status_text stat_text
	
end




def SU2IN::trunc_path(s)
	j=0
	for i in (0..s.length)
		if s[i,1] == "\\" or s[i,1] == "/"
			j=i
		end 
	end
	if j != 0
	s=s[j+1..s.length]
	end
		if s[s.length-3,3]=="jpg" or s[s.length-3,3]=="JPG" or s[s.length-3,3]=="png" or s[s.length-3,3]=="tif"
		else
		s[s.length-3..s.length]="png"
		end

	s=$g_su2in_textures_prefix+$g_su2in_model_name+"\\"+s
	s
end

#### -------- Mesh export ---------- ######

def SU2IN::export_mesh(out)
	
	$parent_mat=[]
	
	if $g_su2in_export_meshes == true
		
		if (Sketchup.active_model.selection.length>0)
			result=UI.messagebox "Export Selected Entities Only?", MB_YESNO
			if result==6
				SU2IN.export_entities(Sketchup.active_model.selection, Geom::Transformation.new)
			else
				SU2IN.export_entities(Sketchup.active_model.entities, Geom::Transformation.new)
			end
		else
			SU2IN.export_entities(Sketchup.active_model.entities, Geom::Transformation.new)
		end
		
		SU2IN.export_faces(out)
		SU2IN.export_fm_faces(out)
		for definition in $component_definitions
			
			out.puts "		<mesh>"
			out.puts "			<name>#{definition[0]}</name>"  #component definition name
			out.print "			<normal_smoothing>true</normal_smoothing>\n" 
			
			out.puts " 			<embedded>"
			for i in (0..3)	
				out.puts "				<expose_uv_set>"
				out.puts "					<index>#{i}</index>"
				out.puts "					<name>uv#{i}</name>"
				out.puts "				</expose_uv_set>"
			end	

			SU2IN.export_component_face(out,definition)
			out.puts "			</embedded>"
			out.puts "		</mesh>"
		end
		for instance in $component_instances
			SU2IN.export_instances(out,instance)
		end
		

	end	
end

###########

def SU2IN::export_faces(export_name)

	for mat in Sketchup.active_model.materials
		if ($g_su2in_materials[mat]!=nil and $g_su2in_materials[mat]!=[])
			SU2IN.export_face(export_name,mat,false)
			$g_su2in_materials[mat]=[]
		end
	end
	if ($g_su2in_materials[nil]!=nil and $g_su2in_materials[nil]!=[])
		SU2IN.export_face(export_name,nil,false)
		$g_su2in_materials[nil]=[]
	end

end

###########



###########
def SU2IN::export_fm_faces(export_name)


		for mat in Sketchup.active_model.materials
			if ($g_su2in_fm_materials[mat]!=nil and $g_su2in_fm_materials[mat]!=[])
				SU2IN.export_face(export_name,mat,true)
				$g_su2in_fm_materials[mat]=[]
			end
		end
		if ($g_su2in_fm_materials[nil]!=nil and $g_su2in_fm_materials[nil]!=[])
			SU2IN.export_face(export_name,nil,true)
			$g_su2in_fm_materials[nil]=[]
		end

end

##### ------------ Exporting entities to an array -------------- ##########

def SU2IN::export_entities(object, trans)

if object.class == Sketchup::ComponentInstance
	
	if (object.definition.get_attribute 'indigo07_settings','inst',false)
		saved=false
		for i in $component_definitions
			saved=true if i[0]==object.definition.name
		end
		if saved==false
			$definition_materials={}
			SU2IN::export_component_def(object,Geom::Transformation.new)
			$component_definitions+=[[object.definition.name,$definition_materials]] #store newly exported definition in array
		end
		t=trans.to_a
		
		scalex=Math.sqrt(t[0]**2+t[1]**2+t[2]**2)
		scaley=Math.sqrt(t[4]**2+t[5]**2+t[6]**2)
		scalez=Math.sqrt(t[8]**2+t[9]**2+t[10]**2)

		#UI.messagebox scalex
		#UI.messagebox scaley
		#UI.messagebox scalez

		
		if ((scalex-scaley).abs<0.000001) and ((scaley-scalez).abs<0.000001)  #check if instance is uniform scale
			$component_instances=$component_instances + [[object,trans]] #store instance object and transformation in array
			return
		else #instance is non-uniform scale
			entity_list=SU2IN.dummy(object)
		end
		
	else #if not an instance
		
		entity_list=SU2IN.dummy(object)

	end


elsif object.class == Sketchup::Group

	entity_list=object.entities
	 
else
	entity_list=object
	
end
text=""
text="Component: " + object.definition.name if object.class == Sketchup::ComponentInstance
text="Group" if object.class == Sketchup::Group

SU2IN.status_bar("Collecting Faces - Level #{$parent_mat.size} - #{text}")  #{$face}

	for e in entity_list
		if (e.class == Sketchup::Group and e.visible? and e.layer.visible?)
			$fm_comp.push(false)

			if e.material != nil
				mat = e.material
				$parent_mat.push(e.material)
				#SU2IN.store_textured_entities(e)			
			else
				$parent_mat.push($parent_mat.last)
			end

			SU2IN.export_entities(e, trans*e.transformation)
			
			$parent_mat.pop
			$fm_comp.pop
		end
		if (e.class == Sketchup::ComponentInstance and e.visible? and e.layer.visible?)

			$fm_comp.push(e.definition.behavior.always_face_camera?)

				if e.material != nil
					mat = e.material
					$parent_mat.push(e.material)
					#SU2IN.store_textured_entities(e)
				else
					$parent_mat.push($parent_mat.last)
				end
			
			if (e.definition.name.downcase.slice("indigosphere")=="indigosphere") 
				$spheres=$spheres + [[e,trans]]
			else
				SU2IN.export_entities(e, trans*e.transformation)
			end

			$parent_mat.pop
			$fm_comp.pop
							
							
		end
		if (e.class == Sketchup::Face and e.visible? and e.layer.visible?)

			if e.material == nil
				mat=$parent_mat.last				
			else
				#SU2IN.store_textured_entities(e)
				mat = e.material
			end

			if $g_su2in_materials[mat] == nil
				$g_su2in_materials[mat] = []
			end

			if $g_su2in_fm_materials[mat] == nil
				$g_su2in_fm_materials[mat] = []
			end


			#if $fm_comp.last==true
				#($g_su2in_fm_materials[mat] ||= []) << [e,trans]
			#	$g_su2in_fm_materials[mat] = $g_su2in_fm_materials[mat] + [[e,trans]]
			#else
				$g_su2in_materials[mat] = $g_su2in_materials[mat] + [[e,trans]] if $animation==false
				#($g_su2in_materials[mat] ||= []) << [e,trans]
			#end

			#$face+=1
		end
	end

return nil
end

def SU2IN::store_textured_entities(e)
	if (e.material.respond_to?(:texture) and e.material.texture !=nil)
		saved=false
		for i in $g_su2in_model_textures
			saved=true if i[0]== e.material
		end
		if saved==false
			$g_su2in_model_textures.push([e.material, e])
		end
	end
end

def SU2IN::point_to_vector(p)
	Geom::Vector3d.new(p.x,p.y,p.z)
end

##### ---- Exporting polymeshes ---- #######

def SU2IN::export_face(out,mat,fm_mat)

	uvset0={}
	uvset1={}
	uvset2={}
	uvset3={}
	meshes = []
	polycount = 0
	pointcount = 0
	mirrored={}
	if fm_mat
		export=$g_su2in_fm_materials[mat]
	else
		export=$g_su2in_materials[mat]
	end

	for ft in export
			
		polymesh = ft[0].mesh 5 #
		polymesh.transform! ft[1]
		mirrored[polymesh] = true
		xa = SU2IN.point_to_vector(ft[1].xaxis)
		ya = SU2IN.point_to_vector(ft[1].yaxis)
		za = SU2IN.point_to_vector(ft[1].zaxis)
		xy = xa.cross(ya)
		xz = xa.cross(za)
		yz = ya.cross(za)
		if xy.dot(za) < 0
			mirrored[polymesh] = !mirrored[polymesh]
		end
		if xz.dot(ya) < 0
			mirrored[polymesh] = !mirrored[polymesh]
		end
		if yz.dot(xa) < 0
			mirrored[polymesh] = !mirrored[polymesh]
		end
		meshes = meshes + [polymesh]

		uvset0[polymesh]=ft[0].get_attribute 'uvs','uv0'
		uvset1[polymesh]=ft[0].get_attribute 'uvs','uv1'
		uvset2[polymesh]=ft[0].get_attribute 'uvs','uv2'
		uvset3[polymesh]=ft[0].get_attribute 'uvs','uv3'	

		polycount=polycount + polymesh.count_polygons
		pointcount=pointcount + polymesh.count_points
	end

	startindex = 0
	has_texture = false
	
	if mat
		matname = mat.display_name
		matname=matname.gsub(/[<>]/, '*') 
		
		#if mat.texture   #testing uv mapping without textures
			has_texture = true
		#end
	else
		matname = "Default"
	end

matname="FM_"+matname if fm_mat

#Exporting verticies

if mat
	SU2IN.status_bar("Material being exported: " + mat.name + " - Verticies")
end

out.puts "		<mesh>"
out.puts "			<name>#{matname}</name>"
if mat
	out.print "			<normal_smoothing>false</normal_smoothing>\n" if ((mat.get_attribute 'indigo07_materials','no_smooth',"No")=="Yes")
	out.print "			<normal_smoothing>true</normal_smoothing>\n" if ((mat.get_attribute 'indigo07_materials','no_smooth',"No")=="No")
else
	out.print "			<normal_smoothing>true</normal_smoothing>\n" 
end

out.puts " 			<embedded>"


for i in (0..3)	
	out.puts "				<expose_uv_set>"
	out.puts "					<index>#{i}</index>"
	out.puts "					<name>uv#{i}</name>"
	out.puts "				</expose_uv_set>"
end	


	for mesh in meshes
		for p in (1..mesh.count_points)
			pos = mesh.point_at(p).to_a
			norm = mesh.normal_at(p).to_a
			if (has_texture and $g_su2in_clay==false)
				if has_texture and mat==nil
					texsize = Geom::Point3d.new(mat.texture.width, mat.texture.height, 1)
				else
					texsize = Geom::Point3d.new(1,1,1)
				end

				uv0=uvset0[mesh][p-1] if uvset0[mesh]
				uv1=uvset1[mesh][p-1] if uvset1[mesh]
				uv2=uvset2[mesh][p-1] if uvset2[mesh]
				uv3=uvset3[mesh][p-1] if uvset3[mesh]
				
				uv = [mesh.uv_at(p,1).x/texsize.x, mesh.uv_at(p,1).y/texsize.y, mesh.uv_at(p,1).z/texsize.z]

				if has_texture 
					out.print "				<vertex pos=\"#{"%.4f" %(pos[0]*$g_su2in_scale)} #{"%.4f" %(pos[1]*$g_su2in_scale)} #{"%.4f" %(pos[2]*$g_su2in_scale)}\" normal=\"#{"%.4f" %(norm[0])} #{"%.4f" %(norm[1])} #{"%.4f" %(norm[2])}\" "
					out.print "uv0=\"#{"%.4f" %(uv[0])} #{"%.4f" %(uv[1]+1)}\" " if uv0==nil
					out.print "uv0=\"#{"%.4f" %(uv0[0])} #{"%.4f" %(uv0[1]+1)}\" " if uv0
					out.print "uv1=\"#{"%.4f" %(uv1[0])} #{"%.4f" %(uv1[1]+1)}\" " if uv1
					out.print "uv2=\"#{"%.4f" %(uv2[0])} #{"%.4f" %(uv2[1]+1)}\" " if uv2
					out.print "uv3=\"#{"%.4f" %(uv3[0])} #{"%.4f" %(uv3[1]+1)}\" " if uv3
					out.print "/>\n"
						
				end
			else
				out.print "				<vertex pos=\"#{"%.4f" %(pos[0]*$g_su2in_scale)} #{"%.4f" %(pos[1]*$g_su2in_scale)} #{"%.4f" %(pos[2]*$g_su2in_scale)}\" normal=\"#{"%.4f" %(norm[0])} #{"%.4f" %(norm[1])} #{"%.4f" %(norm[2])}\"/>\n"
			end
		end
	end

	#Exporting faces

	SU2IN.status_bar("Material being exported: " + mat.name + " - Faces") if mat

			out.print "				<triangle_set>\n"
			if ($g_su2in_clay==true) or ($monochrome==true)
				out.print "					<material_name>#{$monochrome_material}</material_name>\n"
			else
				out.print "					<material_name>#{matname}</material_name>\n"
			end
	for mesh in meshes
		for poly in mesh.polygons
			v1 = (poly[0]>=0?poly[0]:-poly[0])+startindex
			v2 = (poly[1]>=0?poly[1]:-poly[1])+startindex
			v3 = (poly[2]>=0?poly[2]:-poly[2])+startindex
			if !mirrored[mesh]
				out.print "					<tri>#{v1-1} #{v2-1} #{v3-1}</tri>\n"
			else
				out.print "					<tri>#{v2-1} #{v1-1} #{v3-1}</tri>\n"
			end
			$g_su2in_count_tri = $g_su2in_count_tri + 1
		end
		startindex = startindex + mesh.count_points
	end

			out.puts "				</triangle_set>"
			out.puts "			</embedded>"
			out.puts "		</mesh>"


#Exporting Material
	
		
		
		
       $g_su2in_count_material = $g_su2in_count_material + 1
	type=mat.get_attribute 'indigo07_materials','type',"Diffuse" if mat
	
	if (mat != nil and type=="Blackbody Light")
		$n_meshlights+=1
		
		temp=mat.get_attribute 'indigo07_materials','temp',0.0
		power=mat.get_attribute 'indigo07_materials','power'
		ole=mat.get_attribute 'indigo07_materials','ole',17.5
		
		out.print "		<meshlight>\n"
		out.print "			<pos>0.000000 0.000000 0.000000</pos>\n"	
		out.print "			<scale>1.0</scale>\n"	
		out.print "			<rotation>\n"
		out.print "				<matrix>\n"
		out.print "					1.0 0.000000 0.000000 0.000000 1.0 0.000000 0.000000 0.000000 1.0\n"
		out.print "				</matrix>\n"
		out.print "			</rotation>\n"
		out.print " 			<mesh_name>#{matname}</mesh_name>\n"
		out.print " 			<spectrum>\n"
		out.print "				<blackbody>\n"
		out.print " 					<temperature>#{temp}</temperature>\n"
		out.print "					<gain>1.0</gain>\n"
		out.print "				</blackbody>\n"
		out.print "			</spectrum>\n"
		out.print "			<efficacy_scale>\n"
		out.print "				<power_drawn>#{power}</power_drawn>\n"
		out.print "				<overall_luminous_efficacy>#{ole}</overall_luminous_efficacy>\n"
		out.print "			</efficacy_scale>\n"
		out.print "		</meshlight>\n"
		
	end


	#export RGB Meshlight
	
	if (mat != nil and type=="RGB Light")
		$n_meshlights+=1
		
		result=mat.color.red,mat.color.green,mat.color.blue
		power=mat.get_attribute 'indigo07_materials','power'
		ole=mat.get_attribute 'indigo07_materials','ole',17.5
		out.print "		<meshlight>\n"
		out.print "			<pos>0.000000 0.000000 0.000000</pos>\n"	
		out.print "			<scale>1.0</scale>\n"	
		out.print "			<rotation>\n"
		out.print "				<matrix>\n"
		out.print "					1.0 0.000000 0.000000 0.000000 1.0 0.000000 0.000000 0.000000 1.0\n"
		out.print "				</matrix>\n"
		out.print "			</rotation>\n"
		out.print " 			<mesh_name>#{matname}</mesh_name>\n"
		out.print " 			<spectrum>\n"
		out.print "				<rgb>\n"
		out.print " 					<rgb>#{result[0]/255.0**2.2} #{result[1]/255.0**2.2} #{result[2]/255.0**2.2}</rgb>\n"
		out.print "				</rgb>\n"
		out.print "			</spectrum>\n"
		out.print "			<efficacy_scale>\n"
		out.print "				<power_drawn>#{power}</power_drawn>\n"
		out.print "				<overall_luminous_efficacy>#{ole}</overall_luminous_efficacy>\n"
		out.print "			</efficacy_scale>\n"
		out.print "		</meshlight>\n"
		
	end
	if (mat and (type != "RGB Light") and (type != "Blackbody Light"))
		out.print "		<model>\n"
		out.print "			<pos>0.0 0.0 0.0</pos>\n"
		out.print "			<scale>1.0</scale>\n"	
		out.print "			<rotation>\n"
		out.print "				<matrix>\n"
		out.print "					1.0 0.0 0.0 0.0 1.0 0.0 0.0 0.0 1.0 \n"
		out.print "				</matrix>\n"
		out.print "			</rotation>\n"
		out.print "			<mesh_name>#{matname}</mesh_name>\n"
		out.print "		</model>\n"
	end
	if (mat==nil)
		out.print "		<model>\n"
		out.print "			<pos>0.0 0.0 0.0</pos>\n"
		out.print "			<scale>1.0</scale>\n"	
		out.print "			<rotation>\n"
		out.print "				<matrix>\n"
		out.print "					1.0 0.0 0.0 0.0 1.0 0.0 0.0 0.0 1.0 \n"
		out.print "				</matrix>\n"
		out.print "			</rotation>\n"
		out.print "			<mesh_name>#{matname}</mesh_name>\n"
		out.print "		</model>\n"
	end

mirrored={}
meshes = []
uvset0={}
uvset1={}
uvset2={}
uvset3={}

end


##### ---- Exporting Component Definitions polymeshes ---- #######

def SU2IN::export_component_face(out,definition)

material_list=Sketchup.active_model.materials.to_a
material_list=material_list.push(nil) 
startindex = 0
uvset0={}
uvset1={}
uvset2={}
uvset3={}

#########Trying to fix multi-material instance bug
for mat in material_list
	
		if (definition[1][mat]!=nil and definition[1][mat]!=[])
			meshes = []
			mirrored={}
	
	export=definition[1][mat]
	
	for ft in export
		polymesh = ft[0].mesh 5 #
		polymesh.transform! ft[1]
		mirrored[polymesh] = true
		xa = SU2IN.point_to_vector(ft[1].xaxis)
		ya = SU2IN.point_to_vector(ft[1].yaxis)
		za = SU2IN.point_to_vector(ft[1].zaxis)
		xy = xa.cross(ya)
		xz = xa.cross(za)
		yz = ya.cross(za)
		if xy.dot(za) < 0
			mirrored[polymesh] = !mirrored[polymesh]
		end
		if xz.dot(ya) < 0
			mirrored[polymesh] = !mirrored[polymesh]
		end
		if yz.dot(xa) < 0
			mirrored[polymesh] = !mirrored[polymesh]
		end
		meshes = meshes + [polymesh]

		uvset0[polymesh]=ft[0].get_attribute 'uvs','uv0'
		uvset1[polymesh]=ft[0].get_attribute 'uvs','uv1'
		uvset2[polymesh]=ft[0].get_attribute 'uvs','uv2'
		uvset3[polymesh]=ft[0].get_attribute 'uvs','uv3'
		
	end

	
	has_texture = false
	if mat
		matname = mat.display_name
		matname=matname.gsub(/[<>]/, '*') 
		has_texture=true
				
	else
		matname = "Default"
	end

#Exporting verticies


	for mesh in meshes
		for p in (1..mesh.count_points)
			pos = mesh.point_at(p).to_a
			norm = mesh.normal_at(p).to_a
			if (has_texture and $g_su2in_clay==false)
				if has_texture and mat==nil
					texsize = Geom::Point3d.new(mat.texture.width, mat.texture.height, 1)
				else
					texsize = Geom::Point3d.new(1,1,1)
				end
				
				uv0=uvset0[mesh][p-1] if uvset0[mesh]
				uv1=uvset1[mesh][p-1] if uvset1[mesh]
				uv2=uvset2[mesh][p-1] if uvset2[mesh]
				uv3=uvset3[mesh][p-1] if uvset3[mesh]
				
				uv = [mesh.uv_at(p,1).x/texsize.x, mesh.uv_at(p,1).y/texsize.y, mesh.uv_at(p,1).z/texsize.z]
				if has_texture 
					out.print "				<vertex pos=\"#{"%.4f" %(pos[0]*$g_su2in_scale)} #{"%.4f" %(pos[1]*$g_su2in_scale)} #{"%.4f" %(pos[2]*$g_su2in_scale)}\" normal=\"#{"%.4f" %(norm[0])} #{"%.4f" %(norm[1])} #{"%.4f" %(norm[2])}\" "
					out.print "uv0=\"#{"%.4f" %(uv[0])} #{"%.4f" %(uv[1]+1)}\" " if uv0==nil
					out.print "uv0=\"#{"%.4f" %(uv0[0])} #{"%.4f" %(uv0[1]+1)}\" " if uv0
					out.print "uv1=\"#{"%.4f" %(uv1[0])} #{"%.4f" %(uv1[1]+1)}\" " if uv1
					out.print "uv2=\"#{"%.4f" %(uv2[0])} #{"%.4f" %(uv2[1]+1)}\" " if uv2
					out.print "uv3=\"#{"%.4f" %(uv3[0])} #{"%.4f" %(uv3[1]+1)}\" " if uv3
					out.print "/>\n"
						
				end
			else
				out.print "				<vertex pos=\"#{"%.4f" %(pos[0]*$g_su2in_scale)} #{"%.4f" %(pos[1]*$g_su2in_scale)} #{"%.4f" %(pos[2]*$g_su2in_scale)}\" normal=\"#{"%.4f" %(norm[0])} #{"%.4f" %(norm[1])} #{"%.4f" %(norm[2])}\"/>\n"
			end
		end
	end

	#Exporting faces

	SU2IN.status_bar("Material being exported: " + mat.name + " - Faces") if mat

			out.print "				<triangle_set>\n"
			if ($g_su2in_clay==true) or ($monochrome==true)
				out.print "					<material_name>#{$monochrome_material}</material_name>\n"
			else
				out.print "					<material_name>#{matname}</material_name>\n"
			end
	for mesh in meshes
		for poly in mesh.polygons
			v1 = (poly[0]>=0?poly[0]:-poly[0])+startindex
			v2 = (poly[1]>=0?poly[1]:-poly[1])+startindex
			v3 = (poly[2]>=0?poly[2]:-poly[2])+startindex
			if !mirrored[mesh]
				out.print "					<tri>#{v1-1} #{v2-1} #{v3-1}</tri>\n"
			else
				out.print "					<tri>#{v2-1} #{v1-1} #{v3-1}</tri>\n"
			end
			$g_su2in_count_tri = $g_su2in_count_tri + 1
		end
		startindex = startindex + mesh.count_points
	end

			out.puts "				</triangle_set>"

	definition[1][mat]=[] # all verticies and tris of this material have been exported so empty array

	end # end if there are entities stored in this material hash

uvset0={}
uvset1={}
uvset2={}
uvset3={}
			
end # end for each material in definition loop

end # end function


######################################

def SU2IN::export_instances(out,instance)

		mesh_name=instance[0].definition.name
		trans=instance[1]
		
		trans=trans.to_a
		
		
		####################TEST##############
		mat=instance[0].material
		UI.messagebox mat.display_name if mat
		#if mat
		#	temp=mat.name
		#	temp2=mat.display_name
		#	mat.name="Default"
		#	mat.display_name="Default"
		#	SU2IN.export_mat(out,mat,true)
		#	mat.name=temp
		#	mat.display_name=temp2
		#else
		#	SU2IN.export_mat(out,mat,true)
		#end
		
		scalex=Math.sqrt(trans[0]**2+trans[1]**2+trans[2]**2)
		scaley=Math.sqrt(trans[4]**2+trans[5]**2+trans[6]**2)
		scalez=Math.sqrt(trans[8]**2+trans[9]**2+trans[10]**2)

			
		scale=scalex
		scale=scaley if scaley>scale
		scale=scalez if scalez>scale
		
		#SU2IN.export_default_mat(out)
				
		out.print "		<model>\n"
		out.print "			<pos>#{trans[12]*$g_su2in_scale} #{trans[13]*$g_su2in_scale} #{trans[14]*$g_su2in_scale}</pos>\n"
		out.print "			<scale>#{scale}</scale>\n"	
		out.print "			<rotation>\n"
		out.print "				<matrix>\n"
		out.print "					#{trans[0]/scalex} #{trans[4]/scaley} #{trans[8]/scalez} #{trans[1]/scalex} #{trans[5]/scaley} #{trans[9]/scalez} #{trans[2]/scalex} #{trans[6]/scaley} #{trans[10]/scalez}\n"
		out.print "				</matrix>\n"
		out.print "			</rotation>\n"
		out.print "			<mesh_name>#{mesh_name}</mesh_name>\n"
		out.print "		</model>\n"

end


### ------ Function RGB_2_HSB -------- ###
def SU2IN::rgb_to_hsb(r, g, b)

r/=255.0
g/=255.0
b/=255.0

  rgb_max = [r,g,b].max
  rgb_min = [r,g,b].min

  v = rgb_max.to_f

  if rgb_max != 0
    	s = (rgb_max - rgb_min).to_f / (rgb_max).to_f
  else
    	s = 0.0
	h = 0.0
	return h,s,v*100.0
  end

if r==g && g==b
	h=0     
else
    h = ( g-b )/ (rgb_max - rgb_min).to_f if r == rgb_max
    
    h = 2.0 + (b-r).to_f/(rgb_max - rgb_min).to_f if g == rgb_max

    h = 4.0 + (r-g).to_f/(rgb_max - rgb_min).to_f if b == rgb_max
end

h+=6 if h<0
return ((h*600.0).round)/10.0,((s*1000.0).round)/10.0,((v*1000.0).round)/10.0
end


### ------ Function HSB_2_RGB -------- ###

def SU2IN::hsb_to_rgb(h,s,v)
 
	if ((s == 0) && (h == 0))
		r = g = b = v
	end
	v/=100.0
	s/=100.0
	h = 0.0 if (h == 360)
	
	h /= 60.0
	i = h.floor
	f = h - i
	p = v*(1 - s)
	q = v*(1 - s*f)
	t = v*(1 - s*(1 - f))

	case i
	when 0
		r = v
 		g = t
		b = p

	when 1
		r = q
		g = v
		b = p
	when 2
		r = p
		g = v
		b = t
	when 3
		r = p
		g = q
		b = v
	when 4
		r = t
		g = p
		b = v
	when 5
		r = v
		g = p
		b = q
	end

r*=255.0		
g*=255.0
b*=255.0

return r.round,g.round,b.round

end



#############Assigning Indigo Material Properties##########################
def SU2IN::assign_properties_to_material(material)

SU2IN.reset_global_variables
model = Sketchup.active_model

if material==nil
	# Get selected object's material
  	ent=model.selection.first
	if ent and (ent.material!=nil)
		material=ent.material
	end
end

if not material
	UI.messagebox "No material selected.  Select and entity with a material or double-click a material in the Paint Bucket."
	return
end
	
presets=["DIFFUSE","PHONG","SPECULAR","GLOSSY TRANSPARENT","BLEND","METALS","THIN GLASS","Glass","Frosted Glass","Mirror","Ceramic","Dull Plastic","Shiny Plastic","Leather","Car Paint","Curtains","Lampshade","RGB LIGHT","BLACKBODY LIGHT","100W Incandescent","60W Incandescent","7W Compact Fluorescent (warm)","34W Tubular Fluorescent (cold)","34W Tubular Fluorescent (mid)","34W Tubular Fluorescent (warm)","Load IGM","Save IGM"]

#Get Array of Materials with Textures

textures=""
texture_list=["None"]

materials=model.materials
materials.each do |m| 
	if (m.respond_to?(:texture) and m.texture !=nil)
		textures+=(m.display_name)+"|"
	end
next
	
end 

texture_list=textures.split('|') if textures !=""
texture_list.unshift("None") if texture_list != ["None"]

#material=$selected_material if $selected_material

current_mat=material.display_name
current_bump=material.get_attribute 'indigo07_materials','bump_map',"None"
current_exp=material.get_attribute 'indigo07_materials','exp_map',"None"
current_clip=material.get_attribute 'indigo07_materials','clip_map',"None"

# Display a dialog to pick material properties

current_preset=""

if material.alpha<1.0
	current_preset=material.get_attribute 'indigo07_materials','type'
	current_preset=SU2IN.apply_glass(material) if not current_preset  #Sets all transparent materials to glass by default
	if (current_preset=="Al") or (current_preset=="Cr") or (current_preset=="Cu") or (current_preset=="Au") or (current_preset=="Ag") or (current_preset=="Ti") or (current_preset=="W") or (current_preset=="Li") or (current_preset=="Ni") or (current_preset=="Pt")
		current_preset="METALS"
	else
		current_preset=current_preset.upcase
	end
else
	current_preset=material.get_attribute 'indigo07_materials','type'
	current_preset=material.set_attribute 'indigo07_materials','type',"Diffuse" if not current_preset  #Sets all non-transparent materials to diffuse type by default
	if (current_preset=="Al") or (current_preset=="Cr") or (current_preset=="Cu") or (current_preset=="Au") or (current_preset=="Ag") or (current_preset=="Ti") or (current_preset=="W") or (current_preset=="Li") or (current_preset=="Ni") or (current_preset=="Pt")
		current_preset="METALS"
	else
		current_preset=current_preset.upcase
	end
end


if current_preset=="DIFFUSE"
	
	disable_smoothing=%w[Yes No].join("|")
	prompts = ["Set material to","Bump Map", "Bump Gain","Clip Map","Disable Smoothing"]
	values = [current_preset,current_bump , (material.get_attribute 'indigo07_materials','bump_value',0.0),current_clip,(material.get_attribute 'indigo07_materials','no_smooth',"No")]
	enums=[presets.join("|"),texture_list.join("|"),nil,texture_list.join("|"),disable_smoothing]
	results=inputbox prompts,values, enums,(current_mat+" - Diffuse") if material.alpha==1.0
	results=inputbox prompts,values, enums,(current_mat+" - Diffuse (Transmitter)") if material.alpha<1.0
	return if not results
	new_preset=results[0]
	bump_map = results[1]
	bump_value=results[2]
	clip_map=results[3]
	disable_smoothing=results[4]
	type="Diffuse"
	
		
end

if current_preset=="PHONG"
	disable_smoothing=%w[Yes No].join("|")
	prompts = ["Set material to","Index of Refraction (>=1.0)", "Exponent (>=0)", "Fresnel Scale (0-1.0)","Bump Map","Bump Gain","Exponent Map","Exponent Map Gain","Clip Map","Disable Smoothing"]
	values = [current_preset,(material.get_attribute 'indigo07_materials','ior',1.52),(material.get_attribute 'indigo07_materials','exp',1000),(material.get_attribute 'indigo07_materials','fres',1.0),current_bump,(material.get_attribute 'indigo07_materials','bump_value',0.0),current_exp,(material.get_attribute 'indigo07_materials','expmap_value',0.0),current_clip,(material.get_attribute 'indigo07_materials','no_smooth',"No")]
	enums=[presets.join("|"),nil,nil,nil,texture_list.join("|"),nil,texture_list.join("|"),nil,texture_list.join("|"),disable_smoothing]
	results=inputbox prompts,values, enums,(current_mat+" - Phong")
	return if not results
	new_preset=results[0]
	ior = results[1]
	exponent = results[2]
	fresnel = results[3]
	bump_map = results[4]
	bump_value=results[5]
	exp_map=results[6]
	expmap_value=results[7]
	clip_map=results[8]
	disable_smoothing=results[9]
	type="Phong"
end

if current_preset=="SPECULAR"
	disable_smoothing=%w[Yes No].join("|")
	sss=%w[Yes No].join("|")
	prompts = ["Set material to","Index of Refraction (>=1.0)", "Cauchy Dispersion Coefficient (>=0.0)","Precedence (>=1)","Sub-Surface Scattering","Scattering Spectrum","Bump Map","Bump Gain","Clip Map","Disable Smoothing"]
	values = [current_preset,(material.get_attribute 'indigo07_materials','ior',1.52),(material.get_attribute 'indigo07_materials','cau',0.0),(material.get_attribute 'indigo07_materials','precedence',10),(material.get_attribute 'indigo07_materials','sss',"No"),(material.get_attribute 'indigo07_materials','sssval',1000).to_s,current_bump,(material.get_attribute 'indigo07_materials','bump_value',0.0),current_clip,(material.get_attribute 'indigo07_materials','no_smooth',"No")]
	enums = [presets.join("|"),nil,nil,nil,sss,nil,texture_list.join("|"),nil,texture_list.join("|"),disable_smoothing]
	results=inputbox prompts,values,enums, (current_mat+" - Specular")
	return if not results
	new_preset=results[0]
	ior = results[1]
	cauchy = results[2]
	precedence=results[3]
	sss=results[4]
	sss_spectrum=results[5]
	bump_map=results[6]
	bump_value=results[7]
	clip_map=results[8]
	disable_smoothing=results[9]
	type="Specular"

end

if current_preset=="GLOSSY TRANSPARENT"
	disable_smoothing=%w[Yes No].join("|")
	sss=%w[Yes No].join("|")
	prompts = ["Set material to","Index of Refraction (>=1.0)","Exponent (>=0)","Cauchy Dispersion Coefficient (>=0.0)","Precedence (>=1)","Sub-Surface Scattering","Scattering Spectrum","Bump Map","Bump Gain","Clip Map","Disable Smoothing"]
	values = [current_preset,(material.get_attribute 'indigo07_materials','ior',1.52),(material.get_attribute 'indigo07_materials','exp',1000),(material.get_attribute 'indigo07_materials','cau',0.0),(material.get_attribute 'indigo07_materials','precedence',10),(material.get_attribute 'indigo07_materials','sss',"No"),(material.get_attribute 'indigo07_materials','sssval',1000).to_s,current_bump,(material.get_attribute 'indigo07_materials','bump_value',0.0),current_clip,(material.get_attribute 'indigo07_materials','no_smooth',"No")]
	enums = [presets.join("|"),nil,nil,nil,nil,sss,nil,texture_list.join("|"),nil,texture_list.join("|"),disable_smoothing]
	results=inputbox prompts,values,enums, (current_mat+" - Glossy Transparent")
	return if not results
	new_preset=results[0]
	ior = results[1]
	exponent=results[2]
	cauchy = results[3]
	precedence=results[4]
	sss=results[5]
	sss_spectrum=results[6]
	bump_map=results[7]
	bump_value=results[8]
	clip_map=results[9]
	disable_smoothing=results[10]
	type="Glossy Transparent"

end

if current_preset=="BLEND"
	
	disable_smoothing=%w[Yes No].join("|")
	blend_list=["None"]
	for mat in materials
		if ((mat.get_attribute 'indigo07_materials','type',"Diffuse") !="Blackbody Light") and ((mat.get_attribute 'indigo07_materials','type',"Diffuse")!="RGB Light")
			blend_list.push(mat.display_name) if mat!=material
		end
	end
		
	prompts = ["Set material to","Material A", "Material B","Blend Factor (0-1)","Clip Map","Disable Smoothing"]
	values = [current_preset,(material.get_attribute 'indigo_blend','matA',"None"), (material.get_attribute 'indigo_blend','matB',"None"),(material.get_attribute 'indigo_blend','blendfactor',0.5),current_clip,(material.get_attribute 'indigo07_materials','no_smooth',"No")]
	enums=[presets.join("|"),blend_list.join("|"),blend_list.join("|"),nil,texture_list.join("|"),disable_smoothing]
	results=inputbox prompts,values, enums,(current_mat+" - Blend")
	return if not results
	new_preset=results[0]
	material.set_attribute 'indigo_blend','matA',results[1]
	material.set_attribute 'indigo_blend','matB',results[2]
	material.set_attribute 'indigo_blend','blendfactor',results[3]
	clip_map=results[4]
	disable_smoothing=results[5]
	type="Blend"
	
end

if current_preset=="THIN GLASS"
	disable_smoothing=%w[Yes No].join("|")
	prompts = ["Set material to","Exponent (>=0)","Bump Map","Bump Gain","Exponent Map","Exponent Map Gain","Disable Smoothing"]
	values = [current_preset,(material.get_attribute 'indigo07_materials','exp',1000000),current_bump,(material.get_attribute 'indigo07_materials','bump_value',0.0),current_exp,(material.get_attribute 'indigo07_materials','expmap_value',0.0),(material.get_attribute 'indigo07_materials','no_smooth',"No")]
	enums=[presets.join("|"),nil,texture_list.join("|"),nil,texture_list.join("|"),nil,disable_smoothing]
	results=inputbox prompts,values, enums,(current_mat+" - Phong")
	return if not results
	new_preset=results[0]
	exponent = results[1]
	bump_map = results[2]
	bump_value=results[3]
	exp_map=results[4]
	expmap_value=results[5]
	disable_smoothing=results[6]
	type="Thin Glass"
	material.alpha=0.2
	
end

if current_preset=="RGB LIGHT"
	prompts = ["Set material to","Power (Watts)","Overall Luminous Efficacy (lm/W)"]
	values = [current_preset,(material.get_attribute 'indigo07_materials','power',100.0),(material.get_attribute 'indigo07_materials','ole',17.5)]
	enums=[presets.join("|")]
	results=inputbox prompts,values,enums,(current_mat+" - RGB Light")
	return if not results
	new_preset=results[0]
	power=results[1]
	ole=results[2]
	type="RGB Light"
	
		
end

if current_preset=="BLACKBODY LIGHT"
	prompts = ["Set material to","Power (Watts)","Overall Luminous Efficacy (lm/W)","Temperature (Kelvin)"]
	values = [current_preset,(material.get_attribute 'indigo07_materials','power',100.0),(material.get_attribute 'indigo07_materials','ole',17.5),(material.get_attribute 'indigo07_materials','temp',5000)]
	enums=[presets.join("|")]
	results=inputbox prompts,values,enums,(current_mat+" - Blackbody Light")
	return if not results
	new_preset=results[0]
	power=results[1]
	ole=results[2]
	temp=results[3]
	type="Blackbody Light"


end

if (current_preset=="METALS")
	disable_smoothing=%w[Yes No].join("|")
	nk_data=["Aluminum","Chrome","Copper","Gold","Lithium","Nickel","Platinum","Silver","Titanium","Tungsten"]
	nk_list=nk_data.join("|")
	nk_name=["Al","Cr","Cu","Au","Li","Ni","Pt","Ag","Ti","W"]	
	current_type=material.get_attribute 'indigo07_materials','type',"Al"
	index = nk_name.index(current_type)
	index=0 if not index
	
	prompts = ["Set material to","Material","Exponent (>=0)","Bump Map","Bump Gain","Exponent Map","Exponent Map Gain","Clip Map","Disable Smoothing"]
	values = [current_preset,nk_data[index],(material.get_attribute 'indigo07_materials','exp',1000),current_bump,(material.get_attribute 'indigo07_materials','bump_value',0.0),current_exp,(material.get_attribute 'indigo07_materials','expmap_value',0.0),current_clip,(material.get_attribute 'indigo07_materials','no_smooth',"No")]
	enums=[presets.join("|"),nk_list,nil,texture_list.join("|"),nil,texture_list.join("|"),nil,texture_list.join("|"),disable_smoothing]
	results=inputbox prompts,values,enums,(current_mat+" - Metals")
	return if not results
	new_preset=results[0]
	index = nk_data.index(results[1])
	type=nk_name[index]
	exponent = results[2]
	bump_map=results[3]
	bump_value=results[4]
	exp_map=results[5]
	expmap_value=results[6]
	clip_map=results[7]
	disable_smoothing=results[8]
	
	
end

#TODO validate sss spectrum

# Attach the new attributes


material.set_attribute 'indigo07_materials','type', type
material.set_attribute 'indigo07_materials', 'exp', exponent
material.set_attribute 'indigo07_materials', 'fres', fresnel
material.set_attribute 'indigo07_materials', 'ior', ior
material.set_attribute 'indigo07_materials', 'cau', cauchy
material.set_attribute 'indigo07_materials', 'precedence', precedence
material.set_attribute 'indigo07_materials', 'sss', sss
material.set_attribute 'indigo07_materials', 'sssval', sss_spectrum
material.set_attribute 'indigo07_materials', 'power', power
material.set_attribute 'indigo07_materials', 'ole' , ole
material.set_attribute 'indigo07_materials', 'bump_map',bump_map
material.set_attribute 'indigo07_materials', 'bump_value',bump_value
material.set_attribute 'indigo07_materials', 'temp',temp
material.set_attribute 'indigo07_materials', 'no_smooth',disable_smoothing
material.set_attribute 'indigo07_materials', 'exp_map',exp_map
material.set_attribute 'indigo07_materials', 'expmap_value',expmap_value
material.set_attribute 'indigo07_materials', 'clip_map',clip_map

if new_preset!=current_preset

case new_preset
	when "DIFFUSE"
		material.set_attribute 'indigo07_materials','type', "Diffuse"
	when "PHONG"
		material.set_attribute 'indigo07_materials','type', "Phong"
	when "SPECULAR"
		material.set_attribute 'indigo07_materials','type', "Specular"
	when "GLOSSY TRANSPARENT"
		material.set_attribute 'indigo07_materials','type', "Glossy Transparent"
	when "BLEND"
		material.set_attribute 'indigo07_materials','type', "Blend"
	when "METALS"
		material.set_attribute 'indigo07_materials','type', "Al"
	when "THIN GLASS"
		material.set_attribute 'indigo07_materials','type', "Thin Glass"
	when "RGB LIGHT"
		material.set_attribute 'indigo07_materials','type', "RGB Light"
	when "BLACKBODY LIGHT"
		material.set_attribute 'indigo07_materials','type', "Blackbody Light"
   	when "Glass"
		current_preset=SU2IN.apply_glass(material)
	when "Frosted Glass"
		current_preset=SU2IN.apply_frosted_glass(material)
	when "Mirror"
		current_preset=SU2IN.apply_mirror(material)
	when "Ceramic"
		current_preset=SU2IN.apply_ceramic(material)
	when "Dull Plastic"
		current_preset=SU2IN.apply_dull_plastic(material)
	when "Shiny Plastic"
		current_preset=SU2IN.apply_shiny_plastic(material)
	when "Leather"
		current_preset=SU2IN.apply_leather(material)
	when "Car Paint"
		current_preset=SU2IN.apply_car_paint(material)
	when "Curtains"
		current_preset=SU2IN.apply_curtains(material)
	when "Lampshade"
		current_preset=SU2IN.apply_lampshade(material)
	when "100W Incandescent"
		current_preset=SU2IN.apply_100W(material)
	when "60W Incandescent"
		current_preset=SU2IN.apply_60W(material)
	when "7W Compact Fluorescent (Warm)"
		current_preset=SU2IN.apply_7WCFW(material)
	when "34W Tubular Fluorescent (cold)"
		current_preset=SU2IN.apply_34WTFC(material)
	when "34W Tubular Fluorescent (mid)"
		current_preset=SU2IN.apply_34WTFM(material)
	when "34W Tubular Fluorescent (warm)"
		current_preset=SU2IN.apply_34WTFW(material)
	when "Load IGM"
		igm=UI.openpanel "Load IGM", "", "*.igm"
		return if igm==nil
		if (FileTest.exist?(igm)) and (File.basename(igm).slice(".igm")==".igm")
			result=SU2IN.load_igm(igm,material)
			#TODO error reporting
			current_preset=material.get_attribute 'indigo07_materials','type',"Diffuse"
		end
	when "Save IGM"
		
		igm=UI.savepanel("Save IGM", "",((material.display_name.delete("<>"))+".igm"))
		return if igm==nil		
		if igm==igm.split(".")[0] #no extension given
			igm+=".igm"
		end
		SU2IN.save_igm(igm,material)
		
end

SU2IN.assign_properties_to_material(material)

end

#SU2IN.assign_properties_to_material

end # function end


def SU2IN::mat_query

table=""
model = Sketchup.active_model
materials = model.materials

typ=[]
index=0

materials.each do |m| 
        	
	if m.alpha<1.0
		typ[index]=m.get_attribute 'indigo07_materials','type',"Specular"
		typ[index]="Diffuse Transmitter" if typ[index]=="Diffuse"
		else
		typ[index]=(m.get_attribute 'indigo07_materials','type',"Diffuse")
	end 
  
	table+=m.display_name.slice(0,10).ljust(20)+"\t"+typ[index]+"\n"
	index +=1
    next
end
      
       
    # Now display the results
	
	header="MATERIAL NAME\tTYPE\n"
	UI.messagebox header+table,MB_OK,"Indigo Material Types"

end


#############load an exr####################3

def SU2IN::load_exr

SU2IN.reset_global_variables

model=Sketchup.active_model

UI.messagebox "Environment maps must be located in the same directory as the Indigo IGS file."
prompts=["Filename (eg. Sky.exr)","Type","Width","Gain"]
values=[$exr,$exr_type,$exr_width,$exr_gain]
position=%w[Spherical Lat_Long].join("|")
enums=[nil,position,nil,nil]

results=inputbox prompts,values,enums,"Environment Map Properties"
return if not results


$exr=model.set_attribute 'indigo07_settings','exr',results[0]
$exr_type=model.set_attribute 'indigo07_settings','exr_type',results[1]
$exr_width=model.set_attribute 'indigo07_settings','exr_width',results[2].abs
$exr_gain=model.set_attribute 'indigo07_settings','exr_gain',results[3].abs

if ($exr_type=="Spherical") and ($exr_width==0)
	UI.messagebox "Width cannot be zero."
	SU2IN.load_exr
end

end


def SU2IN::quick_export

result=UI.messagebox ("Quick Export? Warning: Only material and camera settings will be exported.", MB_YESNOCANCEL)
return if not result
return if result==2
$settings_only=true if result==6
SU2IN.export

end


#########################################Material Library Functions##################3
def SU2IN::apply_glass(material)
material.set_attribute 'indigo07_materials','type', "Specular"
material.set_attribute 'indigo07_materials', 'fres', 1.0
material.set_attribute 'indigo07_materials', 'ior', 1.52
material.set_attribute 'indigo07_materials', 'cau',0.0
material.set_attribute 'indigo07_materials', 'precedence', 10
material.set_attribute 'indigo07_materials', 'sss', "No"
return "SPECULAR"
end

def SU2IN::apply_frosted_glass(material)
material.set_attribute 'indigo07_materials','type', "Glossy Transparent"
material.set_attribute 'indigo07_materials', 'exp', 1000
material.set_attribute 'indigo07_materials', 'fres', 1.0
material.set_attribute 'indigo07_materials', 'ior', 1.52
material.set_attribute 'indigo07_materials', 'cau', 0.0
material.set_attribute 'indigo07_materials', 'precedence', 10
material.set_attribute 'indigo07_materials', 'sss', "No"
return "GLOSSY TRANSPARENT"
end

def SU2IN::apply_mirror(material)
material.set_attribute 'indigo07_materials','type', "Specular"
material.set_attribute 'indigo07_materials', 'ior', 20.0
material.set_attribute 'indigo07_materials', 'cau', 0.0
material.set_attribute 'indigo07_materials', 'precedence', 10
material.set_attribute 'indigo07_materials', 'sss', "No"
return "SPECULAR"
end

def SU2IN::apply_ceramic(material)
material.set_attribute 'indigo07_materials','type', "Phong"
material.set_attribute 'indigo07_materials', 'exp', 5000
material.set_attribute 'indigo07_materials', 'fres', 1.0
material.set_attribute 'indigo07_materials', 'ior', 1.5
return "PHONG"
end

def SU2IN::apply_dull_plastic(material)
material.set_attribute 'indigo07_materials','type', "Phong"
material.set_attribute 'indigo07_materials', 'exp', 300
material.set_attribute 'indigo07_materials', 'fres', 1.0
material.set_attribute 'indigo07_materials', 'ior', 1.33
return "PHONG"
end

def SU2IN::apply_shiny_plastic(material)
material.set_attribute 'indigo07_materials','type', "Phong"
material.set_attribute 'indigo07_materials', 'exp', 10000
material.set_attribute 'indigo07_materials', 'fres', 1.0
material.set_attribute 'indigo07_materials', 'ior', 1.22
return "PHONG"
end

def SU2IN::apply_leather(material)
material.set_attribute 'indigo07_materials','type', "Phong"
material.set_attribute 'indigo07_materials', 'exp', 50
material.set_attribute 'indigo07_materials', 'fres', 1.0
material.set_attribute 'indigo07_materials', 'ior', 1.58
return "PHONG"
end

def SU2IN::apply_car_paint(material)
material.set_attribute 'indigo07_materials','type', "Phong"
material.set_attribute 'indigo07_materials', 'exp', 30000
material.set_attribute 'indigo07_materials', 'fres', 1.0
material.set_attribute 'indigo07_materials', 'ior', 1.72
return "PHONG"
end

def SU2IN::apply_Al(material)
material.set_attribute 'indigo07_materials','type', "Al"
end

def SU2IN::apply_Cr(material)
material.set_attribute 'indigo07_materials','type', "Cr"
end

def SU2IN::apply_Cu(material)
material.set_attribute 'indigo07_materials','type', "Cu"
end

def SU2IN::apply_Au(material)
material.set_attribute 'indigo07_materials','type', "Au"
end

def SU2IN::apply_Ag(material)
material.set_attribute 'indigo07_materials','type', "Ag"
end

def SU2IN::apply_Ti(material)
material.set_attribute 'indigo07_materials','type', "Ti"
end

def SU2IN::apply_W(material)
material.set_attribute 'indigo07_materials','type', "W"
end

def SU2IN::apply_Li(material)
material.set_attribute 'indigo07_materials','type', "Li"
end

def SU2IN::apply_Ni(material)
material.set_attribute 'indigo07_materials','type', "Ni"
end

def SU2IN::apply_Pt(material)
material.set_attribute 'indigo07_materials','type', "Pt"
end

def SU2IN::apply_curtains(material)
material.set_attribute 'indigo07_materials','type', "Diffuse"
material.alpha=0.9
return "DIFFUSE"
end

def SU2IN::apply_lampshade(material)
material.set_attribute 'indigo07_materials','type', "Diffuse"
material.alpha=0.5
return "DIFFUSE"
end

def SU2IN::apply_100W(material)
material.set_attribute 'indigo07_materials','type', "Blackbody Light"
material.set_attribute 'indigo07_materials', 'power',100.0
material.set_attribute 'indigo07_materials', 'ole' , 17.5
material.set_attribute 'indigo07_materials', 'temp', 2800
return "BLACKBODY LIGHT"
end

def SU2IN::apply_60W(material)
material.set_attribute 'indigo07_materials','type', "Blackbody Light"
material.set_attribute 'indigo07_materials', 'power', 60.0
material.set_attribute 'indigo07_materials', 'ole' , 14.0
material.set_attribute 'indigo07_materials', 'temp', 2800
return "BLACKBODY LIGHT"
end

def SU2IN::apply_7WCFW(material)
material.set_attribute 'indigo07_materials','type', "Blackbody Light"
material.set_attribute 'indigo07_materials', 'power',7.0
material.set_attribute 'indigo07_materials', 'ole' , 50.0
material.set_attribute 'indigo07_materials', 'temp', 3000
return "BLACKBODY LIGHT"
end

def SU2IN::apply_34WTFC(material)
material.set_attribute 'indigo07_materials','type', "Blackbody Light"
material.set_attribute 'indigo07_materials', 'power',34.0
material.set_attribute 'indigo07_materials', 'ole' , 50.0
material.set_attribute 'indigo07_materials', 'temp', 6000
return "BLACKBODY LIGHT"
end

def SU2IN::apply_34WTFM(material)
material.set_attribute 'indigo07_materials','type', "Blackbody Light"
material.set_attribute 'indigo07_materials', 'power',34.0
material.set_attribute 'indigo07_materials', 'ole' , 50.0
material.set_attribute 'indigo07_materials', 'temp', 4500
return "BLACKBODY LIGHT"
end

def SU2IN::apply_34WTFW(material)
material.set_attribute 'indigo07_materials','type', "Blackbody Light"
material.set_attribute 'indigo07_materials', 'power',34.0
material.set_attribute 'indigo07_materials', 'ole' , 50.0
material.set_attribute 'indigo07_materials', 'temp', 3000
return "BLACKBODY LIGHT"
end

#####Export Sphere Primitives############

def SU2IN::export_spheres(out)

for sphere in $spheres
	
	inst=sphere[0]
	sphere_mat=inst.material
	if sphere_mat
		sphere_mat_name=sphere_mat.display_name.gsub(/[<>]/, '*')
	else
		sphere_mat_name="Default"
	end
	

	box=inst.bounds
	center=box.center
	center.transform! sphere[1] #get the center of the sphere in world coordinates
	cx=center.x.to_m
	cy=center.y.to_m
	cz=center.z.to_m

	r=box.height.to_m/2 #set radius of sphere equal to the half the height of its bounding box
	
	
	
	
	out.puts "	<sphere>"
	out.puts "		<center>#{cx} #{cy} #{cz}</center>"
	out.puts "		<radius>#{r}</radius>"
	out.puts "		<material_name>#{sphere_mat_name}</material_name>"
	out.puts "	</sphere>"

end



end


##### ------------ Exporting instanced entities to an array -------------- ##########

def SU2IN::export_component_def(object, trans)


if object.class == Sketchup::ComponentInstance

	entity_list=SU2IN.dummy(object)

elsif object.class == Sketchup::Group

	entity_list=object.entities 
else
	entity_list=object
end
text=""
text="Component: " + object.definition.name if object.class == Sketchup::ComponentInstance
text="Group" if object.class == Sketchup::Group

SU2IN.status_bar("Collecting Faces - Level #{$parent_mat.size} - #{text}")  #{$face}

	for e in entity_list
		if (e.class == Sketchup::Group and e.visible? and e.layer.visible?)
			
			if e.material != nil
				mat = e.material
				$parent_mat.push(e.material)
				#SU2IN.store_textured_entities(e)			
			else
				$parent_mat.push($parent_mat.last)
			end

			SU2IN.export_component_def(e, trans*e.transformation)
			
			$parent_mat.pop
			
		end
		if (e.class == Sketchup::ComponentInstance and e.visible? and e.layer.visible?)
			

				if e.material != nil
					mat = e.material
					$parent_mat.push(e.material)
					#SU2IN.store_textured_entities(e)
				else
					$parent_mat.push($parent_mat.last)
				end
			
			if (e.definition.name.downcase.slice("indigosphere")=="indigosphere") 
				$spheres=$spheres + [[e,trans]]
			else
				SU2IN.export_component_def(e, trans*e.transformation)
			end
			
			$parent_mat.pop
			
				
		end
		if (e.class == Sketchup::Face and e.visible? and e.layer.visible?)

			if e.material == nil
				mat=$parent_mat.last				
			else
				#SU2IN.store_textured_entities(e)
				mat = e.material
			end

			if $definition_materials[mat] == nil
				$definition_materials[mat] = []
			end

			$definition_materials[mat] = $definition_materials[mat] + [[e,trans]] if $animation==false
			#($definition_materials[mat] ||= []) << [e,trans]

		end
	end

return
end

###############################################################

def SU2IN::get_indigo_path

path=File.dirname(__FILE__)+"\/indigo_08_path.txt"

if FileTest.exist?(path)  #check if indigo_path.txt exists
	path_file=File.new(path,"r")
	indigo_path=path_file.read  #get the contents of indigo_path.txt file
	path_file.close
		
	if (FileTest.exist?(indigo_path)) and (File.basename(indigo_path)=="indigo.exe") #check if the path to indigo is valid
				
		return indigo_path
	else
		indigo_path=UI.openpanel("Locate indigo.exe","","*.exe")
		path_file=File.new(path,"w") if indigo_path
		path_file.write(indigo_path) if indigo_path
		path_file.close if indigo_path
		
	end
else
	
	indigo_path=UI.openpanel("Locate indigo.exe","","*.exe")
	path_file=File.new(path,"w") if indigo_path
	path_file.write(indigo_path) if indigo_path
	path_file.close if indigo_path
end

if (FileTest.exist?(indigo_path)) and (File.basename(indigo_path)=="indigo.exe") #check if the path to indigo is valid
	return indigo_path
else
	return nil
end

end

##################################################################

def SU2IN::export_textures


tw=Sketchup.create_texture_writer
model = Sketchup.active_model
entities = model.active_entities

helper_group=entities.add_group
materials=model.materials

for mat in materials
	if (mat.respond_to?(:texture) and mat.texture !=nil)
		helper_group.material=mat
		tw.load(helper_group)
	end

end

entities.erase_entities helper_group

status=tw.write_all $g_su2in_path_textures+"\\"+$g_su2in_textures_prefix+$g_su2in_model_name,false
tw=nil

return status

end

####################################################################
def SU2IN::export_igm_textures(igm,mat)

textures=[]				#Array of materials to be used as maps for material being exported
tw=Sketchup.create_texture_writer
model = Sketchup.active_model
entities = model.active_entities
materials=model.materials
translatednames = materials.collect {|m| m.display_name}
names=materials.collect {|m| m.name}

helper_group = entities.add_group

type=mat.get_attribute 'indigo07_materials','type',"Diffuse"
bump_map=mat.get_attribute 'indigo07_materials','bump_map',"None"
exp_map=mat.get_attribute 'indigo07_materials','exp_map',"None"
clip_map=mat.get_attribute 'indigo07_materials','clip_map',"None"


bump_index=translatednames.index(bump_map)
exp_index=translatednames.index(exp_map)
clip_index=translatednames.index(clip_map)

bump_material=bump_index ? materials[names[bump_index]] : nil
exp_material=exp_index ? materials[names[exp_index]] : nil
clip_material=clip_index ? materials[names[clip_index]] : nil

#UI.messagebox type
case type

when ("Diffuse"  or "Diffuse Transmitter")
	if mat.respond_to?(:texture) and (mat.texture !=nil)
		#UI.messagebox "pushing texture for diffuse"
		textures.push(mat)
	end
	if bump_material.respond_to?(:texture) and bump_material.texture !=nil
		textures.push(bump_material)
	end
	if clip_material.respond_to?(:texture) and clip_material.texture !=nil
		textures.push(clip_material)
	end
when "Phong"
	if mat.respond_to?(:texture) and (mat.texture !=nil)
		textures.push(mat)
	end
	if bump_material.respond_to?(:texture) and bump_material.texture !=nil
		textures.push(bump_material)
	end
	if exp_material.respond_to?(:texture) and exp_material.texture !=nil
		textures.push(exp_material)
	end
	if clip_material.respond_to?(:texture) and clip_material.texture !=nil
		textures.push(clip_material)
	end
when "Specular"
	if bump_material.respond_to?(:texture) and bump_material.texture !=nil
		textures.push(bump_material)
	end
	if clip_material.respond_to?(:texture) and clip_material.texture !=nil
		textures.push(clip_material)
	end
when  "Glossy Transparent"
	if bump_material.respond_to?(:texture) and bump_material.texture !=nil
		textures.push(bump_material)
	end
	if exp_material.respond_to?(:texture) and exp_material.texture !=nil
		textures.push(exp_material)
	end
	if clip_material.respond_to?(:texture) and clip_material.texture !=nil
		textures.push(clip_material)
	end
when "Al","Cr","Cu","Au","Li","Ni","Pt","Ag","Ti","W"
	if bump_material.respond_to?(:texture) and bump_material.texture !=nil
		textures.push(bump_material)
	end
	if exp_material.respond_to?(:texture) and exp_material.texture !=nil
		textures.push(exp_material)
	end
	if clip_material.respond_to?(:texture) and clip_material.texture !=nil
		textures.push(clip_material)
	end
when "Blend"
	if mat.respond_to?(:texture) and (mat.texture !=nil)
		textures.push(mat)
	end
	if clip_material.respond_to?(:texture) and clip_material.texture !=nil
		textures.push(clip_material)
	end
end  									#End case type


for map in textures
	helper_group.material=map
	tw.load(helper_group)
end
entities.erase_entities helper_group

status=tw.write_all File.dirname(igm),false
tw=nil

return status

end

####################################Fog Export#########################################
def SU2IN::export_fog(out)

model=Sketchup.active_model
options=model.rendering_options
model_extents=model.bounds
eye=model.active_view.camera.eye
model_extents.add eye #ensures that the camera is inside the fog sphere


center=model_extents.center

cx=center.x.to_m
cy=center.y.to_m
cz=center.z.to_m

depth=model_extents.depth.to_m
height=model_extents.height.to_m
width=model_extents.width.to_m

d=depth
d=height if height>depth
d=width if (width>height) and (width>depth)

r=d/2 #fog sphere radius

fog_end=options["FogEndDist"]
fog_end=fog_end*$g_su2in_scale = 0.0254
fog_color=options["FogColor"]
alpha_factor=1

sss=2.772/fog_end  #distance at which light intensity is halved is 0.693/scattering coefficient

out.puts "		<medium>"
out.print "			<name>Fog_medium</name>\n"
out.print "			<precedence>2</precedence>\n"
out.print "			<basic>\n"
out.print "				<ior>1.0</ior>\n"
out.print "				<cauchy_b_coeff>0.0</cauchy_b_coeff>\n"
out.puts "				<absorption_coefficient_spectrum>"
out.puts "					<rgb>"
#out.print "						<rgb>0.0 0.0 0.0</rgb>\n"
out.print "						<rgb>#{(1.0-fog_color.red/255.0)*alpha_factor} #{(1.0-fog_color.green/255.0)*alpha_factor} #{(1.0-fog_color.blue/255.0)*alpha_factor}</rgb>\n"
out.print "					</rgb>\n"
out.print "				</absorption_coefficient_spectrum>\n"
out.print "				<subsurface_scattering>\n"
out.print "					<scattering_coefficient_spectrum>\n"
out.print "						<uniform>\n"
out.print "							<value>#{sss}</value>\n"
out.print "						</uniform>\n"
out.print "					</scattering_coefficient_spectrum>\n"
out.print "					<phase_function>\n"
out.print "						<uniform/>\n"
out.print "					</phase_function>\n"
out.print "				</subsurface_scattering>\n"
out.print "			</basic>\n"
out.print "		</medium>\n"
out.print "		<material>\n"
out.print " 		<name>Fog</name>\n"
out.print " 		<specular>\n"
out.print " 			<transparent>true</transparent>\n"
out.print "			<internal_medium_name>Fog_medium</internal_medium_name>\n"
out.print " 		</specular>\n"
out.print " 		</material>\n"

out.puts "		<sphere>"
out.puts "			<center>#{cx} #{cy} #{cz}</center>"
out.puts "			<radius>#{r}</radius>"
out.puts "			<material_name>Fog</material_name>"
out.puts "		</sphere>"

end

def SU2IN::parse(tag,string)

data=string.split("<"+tag+">")[1]


data=string.split("<"+tag+">")[2] if tag=="rgb"
if data
	data=data.split("<\/"+tag+">")[0]
	return nil if not data
else
	return nil
end
return data

end

##################################Function to acquire array of media from IGM file##################
def SU2IN::get_media(data)

media_arr=[]
media=data.split("<medium>")
if media.length>1 #we have at least one medium
	for i in (1..media.length-1)
		media_data=media[i].split("</medium>")[0]
		medium_name=parse("name",media_data) if media_data
		media_arr.push(medium_name) if medium_name
		media_arr.push(media_data) if media_data
		
	end
else
	return nil
end

return media_arr

end


##################################Function to acquire array of media from IGM file##################
def SU2IN::get_materials(data)

material_arr=[]
materials=data.split("<material>")

if materials.length>1 #we have at least one material
	for i in (1..materials.length-1)
		material_data=materials[i].split("</material>")[0]
		material_arr.push(material_data) if material_data
	end
else
	return nil
end

return material_arr

end
##################################Function to Load IGM Files##################
def SU2IN::load_igm(igm,mat)

model=Sketchup.active_model
model.start_operation "Load IGM"
media=[]
materials=[]
null_mat=[]
num_materials=0
igm_file=File.new(igm,"r")
md=igm_file.read
igm_file.close
material_data=parse("materialdata",md)
return 0 if not material_data
version=parse("version",material_data)
if version!="0.8"
	UI.messagebox "Warning: IGM was created for an unknown Indigo version and may be incompatible." if not version
	UI.messagebox "Warning: IGM was created for Indigo version #{version} and may be incompatible." if version
end

media=SU2IN.get_media(material_data) #returns hash of media
materials=SU2IN.get_materials(material_data) #returns array of materials

for material in materials
	num_materials=num_materials+1
	material_name=specular=transparent=imn=diffuse=colour=diffuse_transmitter=phong=ior=exponent=fresnel=nk_data=glossy_transparent=blend=a_name=b_name=blend_factor=medium=precedence=basic=cauchy=acs=acs_uniform=acs_uniform_value=acs_rgb_value=sss=scs=scs_uniform=scs_uniform_value=scs_rgb_value=albedo=albedo_path=bump_map=bump_map_path=bump_map_b=exp_map=exp_map_path=exp_map_b=blend_map=blend_map_path=bump_name=bump_mat=bump_value=exp_name=exp_mat=expmap_value=nil
	material_name=parse("name",material)
	#UI.messagebox material_name
	if (material.slice("<null_material\/>")=="<null_material\/>")
		#UI.messagebox "#{material_name} is null"
		null_mat.push(material_name)
		next
	end 
	specular=parse("specular",material)
	if specular
		transparent=parse("transparent",specular)
		imn=parse("internal_medium_name",specular)
	end
	diffuse=parse("diffuse",material)
	if diffuse
		colour=parse("colour",diffuse)
	end
	diffuse_transmitter=parse("diffuse_transmitter",material)
	if diffuse_transmitter
		colour=parse("colour",diffuse_transmitter)
	end
	phong=parse("phong",material)
	if phong
		colour=parse("diffuse",phong)
		ior=parse("ior",phong)
		exponent=parse("exponent",phong)
		fresnel=parse("fresnel_scale",phong)
		nk_data=parse("nk_data",phong)
	end
	glossy_transparent=parse("glossy_transparent",material)
	if glossy_transparent
		imn=parse("internal_medium_name",glossy_transparent)
		exponent=parse("exponent",glossy_transparent)
	end
	blend=parse("blend",material)
	if blend
		a_name=parse("a_name",blend)
		b_name=parse("b_name",blend)
		blend_factor=parse("blend_factor",blend)
		
	end
	
	if (specular) or (glossy_transparent)
		medium_index=media.index(imn)
		medium=media[medium_index+1] if medium_index
		if medium
			precedence=parse("precedence",medium)
			basic=parse("basic",medium)
			if basic
				ior=parse("ior",basic)
				cauchy=parse("cauchy_b_coeff",basic)
				acs=parse("absorption_coefficient_spectrum",basic)
				if acs
					acs_uniform=parse("uniform",acs)
					if acs_uniform
						acs_uniform_value=parse("value",acs_uniform)
					end
					acs_rgb_value=parse("rgb",acs)
				end
			end
			sss=parse("subsurface_scattering",medium)
			if sss
				scs=parse("scattering_coefficient_spectrum",sss)
				if scs
					scs_uniform=parse("uniform",scs)
					if scs_uniform
						scs_uniform_value=parse("value",scs_uniform)
					end
					scs_rgb_value=parse("rgb",scs)
				end
			end

		end #if medium

	end #if specular or glossy transparent


	albedo=parse("albedo_texture",material)
	if albedo
		albedo_path=parse("path",albedo)
	end
	bump_map=parse("bump_map",material)
	if bump_map
		bump_map_path=parse("path",bump_map)
		bump_map_b=parse("b",bump_map)
	end
	exp_map=parse("exponent_map",material)
	if exp_map
		exp_map_path=parse("path",exp_map)
		exp_map_b=parse("b",exp_map)
	end
	blend_map=parse("blend_map",material)
	if blend_map
		blend_map_path=parse("path",blend_map)
	end


########Create new map materials if necessary#######

mats=Sketchup.active_model.materials
#UI.messagebox materials.length
#UI.messagebox num_materials
if (materials.length)>num_materials
	new_mat=mats.add(material_name)
	UI.messagebox "New material created named #{material_name}."
else
	new_mat=mat
end

if albedo_path   							#Case 1 - material has albedo texture
	if FileTest.exist?(File.dirname(igm)+"\\"+albedo_path)
		new_mat.texture=(File.dirname(igm)+"\\"+albedo_path)
		UI.messagebox "Material texture set to #{albedo_path}"
	else
		UI.messagebox "Albedo texture #{albedo_path} not found."
	end
	
end
############################################
if (bump_map_path) and (bump_map_path!=albedo_path)			#Case 2 - material has bump map but not the same as albedo
	if FileTest.exist?(File.dirname(igm)+"\\"+bump_map_path)
		bump_name=new_mat.display_name+"_BUMP"
		bump_mat=mats.add(bump_name)
		bump_name=bump_mat.display_name
		bump_mat.texture=(File.dirname(igm)+"\\"+bump_map_path)
		UI.messagebox "New bump map material created named #{bump_name} with texture #{bump_map_path}."
	else
		UI.messagebox "Bump map #{bump_map_path} not found."
	end
	bump_value=bump_map_b if bump_map_b	
elsif (bump_map_path) and (bump_map_path==albedo_path)			#Case 3 - has bump and is the same as albedo
	bump_name=new_mat.display_name
	UI.messagebox "Bump map set to #{bump_map_path}"
	bump_value=bump_map_b if bump_map_b
end
############################################
if (exp_map_path) and (exp_map_path!=albedo_path)			#Case 4 - has exponent map and it is not the same as albedo
	if FileTest.exist?(File.dirname(igm)+"\\"+exp_map_path)
		exp_name=new_mat.display_name+"_EXP"
		exp_mat=mats.add(exp_name)
		exp_name=exp_mat.display_name			
		exp_mat.texture=(File.dirname(igm)+"\\"+exp_map_path) 
		UI.messagebox "New exponent map material created named #{exp_name} with texture #{exp_map_path}."
	else
		UI.messagebox "Exponent map #{exp_map_path} not found."
	end
	expmap_value=exp_map_b if exp_map_b
elsif (exp_map_path) and (exp_map_path==albedo_path)			#Case 5 - has exp map and is the same as albedo
	exp_name=new_mat.display_name
	UI.messagebox "Exponent map set to #{exp_map_path}"
	expmap_value=exp_map_b if exp_map_b
end
############################################
if (blend_map_path)							#Case 6 - has blend map
	if FileTest.exist?(File.dirname(igm)+"\\"+blend_map_path)
		new_mat.texture=(File.dirname(igm)+"\\"+blend_map_path)
		UI.messagebox "Material texture set to #{blend_map_path}"
	else
		UI.messagebox "Blend Map #{blend_map_path} not found."
	end
end

#######Extract and set RGB Values#############
if (colour) and (new_mat.texture==nil) 			#We have a Diffuse or Phong material

begin
	rgb=colour.split(" ")
	red=((rgb[0].to_f)*255).round
	green=((rgb[1].to_f)*255).round
	blue=((rgb[2].to_f)*255).round
	new_mat.color=[red,green,blue]
	UI.messagebox "Material RGB values set to R=#{red} G=#{green} B=#{blue}"
rescue
	UI.messagebox "Error setting RGB values.  Check IGM file for errors."
end
end #end if colour

#######Extract and set opacity values#########
if (transparent=="true") or glossy_transparent				#We have a material with an internal medium so extract the absorptivity
begin
	if acs_rgb_value
		
		rgb_ab=acs_rgb_value.split(" ")
		red_ab=rgb_ab[0].to_f
		green_ab=rgb_ab[1].to_f
		blue_ab=rgb_ab[2].to_f
		opacity=(Math.sqrt([red_ab,green_ab,blue_ab].max*10)).round
		opacity=99 if opacity>99
		opacity=1 if opacity<1
		red=((1-(10*red_ab)/(opacity**2))*255).round
		red=0 if red<0
		red=255 if red>255
		blue=((1-(10*blue_ab)/(opacity**2))*255).round
		blue=0 if blue<0
		blue=255 if blue>255
		green=((1-(10*green_ab)/(opacity**2))*255).round
		green=0 if green<0
		green=255 if green>255
		set_alpha=opacity.to_f/100
		new_mat.alpha=set_alpha
		new_mat.color=[red,green,blue]
		UI.messagebox "Input Absorption Coefficient Spectrum:R=#{red_ab} G=#{green_ab} B=#{blue_ab}\nSketchup opacity set to #{opacity}\nSketchup RGB set to #{red} #{green} #{blue}\nExported Absorption Coefficient Spectrum:R=#{"%.4f" %((1-red.to_f/255)*opacity*opacity/10)} G=#{"%.4f" %((1-green.to_f/255)*opacity*opacity/10)} B=#{"%.4f" %((1-blue.to_f/255)*opacity*opacity/10)}"
		
	elsif acs_uniform_value
		
		opacity=Math.sqrt(acs_uniform_value.to_f*10).round
		opacity=99 if opacity>99
		opacity=1 if opacity<1
		new_color=((1-(10*acs_uniform_value.to_f)/(opacity**2))*255).round
		new_color=0 if new_color<0
		new_color=255 if new_color>255
		set_alpha=opacity.to_f/100
		new_mat.alpha=set_alpha
		new_mat.color=[new_color,new_color,new_color]
		UI.messagebox "Input Absorption Coefficient Spectrum:Uniform=#{acs_uniform_value}\nSketchup opacity set to #{opacity}\nSketchup RGB set to #{new_color} #{new_color} #{new_color}\nExported Absorption Coefficient Spectrum:R=#{"%.4f" %((1-new_color.to_f/255)*opacity*opacity/10)} G=#{"%.4f" %((1-new_color.to_f/255)*opacity*opacity/10)} B=#{"%.4f" %((1-new_color.to_f/255)*opacity*opacity/10)}"
		
	end		
	
rescue
	UI.messagebox "Error setting opacity and RGB values.  Check IGM file for errors."
end

end #end if transparent

########Apply properties to material#############
if (diffuse)
	new_mat.alpha=1.0
	type="Diffuse"
end
type="Blend" if blend
type="Specular" if specular
type="Glossy Transparent" if glossy_transparent
if phong
	if nk_data
		type=File.basename(nk_data,".nk")
		if (type=="Al") or (type=="Cr") or (type=="Cu") or (type=="Au") or (type=="Ag") or (type=="Ti") or (type=="W") or (type=="Li") or (type=="Ni") or (type=="Pt") 
			new_mat.alpha=1.0
		else
			UI.messagebox "NK Data #{type} is currently not supported by SkIndigo."
			type="Phong"
		end
	else
		type="Phong"
	end
end

if diffuse_transmitter
	new_mat.alpha=0.0
	type="Diffuse"
end
type="Diffuse" if not type
exponent=1000 if not exponent
fresnel=1.0 if not fresnel
ior=1.52 if not ior
cauchy=0.0 if not cauchy
precedence=10 if not precedence
if sss
	sss="Yes"
else
	sss="No"
end
sssval=1000 if not sssval
bump_name="None" if not bump_name
bump_value=0.0 if not bump_value
exp_name="None" if not exp_name
expmap_value=0.0 if not expmap_value
if a_name
	a_name="None" if null_mat.index(a_name)
else
	a_name="None"
end
if b_name
	b_name="None" if null_mat.index(b_name)
else
	b_name="None"
end
blend_factor=0.5 if not blend_factor

new_mat.set_attribute 'indigo07_materials','type', type
new_mat.set_attribute 'indigo07_materials', 'exp', exponent
new_mat.set_attribute 'indigo07_materials', 'fres', fresnel
new_mat.set_attribute 'indigo07_materials', 'ior', ior
new_mat.set_attribute 'indigo07_materials', 'cau', cauchy
new_mat.set_attribute 'indigo07_materials', 'precedence', precedence
new_mat.set_attribute 'indigo07_materials', 'sss', sss
new_mat.set_attribute 'indigo07_materials', 'sssval', scs_uniform_value if scs_uniform_value
new_mat.set_attribute 'indigo07_materials', 'sssval', scs_rgb_value if scs_rgb_value
new_mat.set_attribute 'indigo07_materials', 'bump_map',bump_name
new_mat.set_attribute 'indigo07_materials', 'bump_value',bump_value
new_mat.set_attribute 'indigo07_materials', 'exp_map',exp_name
new_mat.set_attribute 'indigo07_materials', 'expmap_value',expmap_value
new_mat.set_attribute 'indigo_blend', 'matA',a_name
new_mat.set_attribute 'indigo_blend', 'matB',b_name
new_mat.set_attribute 'indigo_blend', 'blendfactor',blend_factor

end #end for mat in materials
model.commit_operation

end #end load igm

################################################################################
def SU2IN::save_igm(igm,mat)

out=File.new(igm,"w")

out.puts "<materialdata>"
out.puts "<version>0.8</version>"

status=SU2IN.save_mat(igm,mat,out)

out.puts "</materialdata>"
out.close

status=SU2IN.export_igm_textures(igm,mat)

stext="Material saved to #{igm}"
UI.messagebox stext

end

###############################################################################
def SU2IN::save_mat(igm,mat,out)

type=mat.get_attribute 'indigo07_materials','type',"Diffuse"
if ((type=="Blackbody Light") or (type=="RGB Light")) and igm
	UI.messagebox "Saving of #{type} materials is currently not supported."
	return
end

if type=="Blend"
	model = Sketchup.active_model
	materials=model.materials
	translatednames = materials.collect {|m| m.display_name}
	names=materials.collect {|m| m.name}
	a_name=mat.get_attribute 'indigo_blend','matA',"None"
	b_name=mat.get_attribute 'indigo_blend','matB',"None"
	a_index=translatednames.index(a_name)
	b_index=translatednames.index(b_name)
	a_mat=a_index ? materials[names[a_index]] : nil
	b_mat=b_index ? materials[names[b_index]] : nil
	SU2IN.save_mat(igm,a_mat,out) if a_mat
	SU2IN.save_mat(igm,b_mat,out) if b_mat
end

if igm
	status=SU2IN.export_igm_textures(igm,mat)
	if not $exported_mats.index(mat)
		SU2IN.export_mat(out,mat,false) if not $exported_mats.index(mat)
		$exported_mats.push(mat)
	end
else
	if not $exported_mats.index(mat)
		SU2IN.export_mat(out,mat,true) if not $exported_mats.index(mat)
		$exported_mats.push(mat)
	end

end

end
####################Autorun Batch File#########################

def SU2IN::create_batch_file(path)

batch_file_path=path[0..path.length-11]+"autorender.bat"
batch=File.new(batch_file_path,"w")

batch.puts "start \/low indigo.exe \"#{$g_su2in_export_file}\"" if $threads=="Low Priority"
batch.puts "start indigo.exe \"#{$g_su2in_export_file}\"" if $threads!="Low Priority"

batch.close
return batch_file_path

end


##################dummy instancing function#############
def SU2IN::dummy(object)

arr=object.definition.name.split("_")
if (arr.last.upcase=="DUMMY") and (arr.length>1)
	arr.pop
	dummy_def_name=arr.join("_")
	if dummy_def_name
		def_list=Sketchup.active_model.definitions
		dummy_def=def_list[dummy_def_name]
	end
end
if dummy_def
	return dummy_def.entities
else
	return object.definition.entities
end

end #function end

end ##########################################SU2IN Class end############################################


#---------------------------------------Custom Tool to select Focal distance----------------------
class FocusTool


def initialize
    @ip1 = nil
        
end


def activate
    
    @ip1 = Sketchup::InputPoint.new
    @ip = Sketchup::InputPoint.new
      
    Sketchup::set_status_text "Enter 0 for Autofocus", SB_VCB_LABEL
    
    self.reset(nil)
end


def deactivate(view)
    view.invalidate 
end


def onMouseMove(flags, x, y, view)
    
        
	@ip.pick view, x, y
        if( @ip != @ip1 )
           
            view.invalidate if( @ip.display? or @ip1.display? )
            @ip1.copy! @ip
                        
            view.tooltip = @ip1.tooltip
        end

end


def onLButtonDown(flags, x, y, view)
    
    
	
        @ip1.pick view, x, y
        if( @ip1.valid? )

            	model=Sketchup.active_model		
		$focal_point=model.set_attribute 'indigo07_settings','fp',@ip1.position
		UI.messagebox ("New Focal Point set to #{$focal_point}")
		                    
        end
                
    
    view.lock_inference
	Sketchup.active_model.select_tool nil
end


def onUserText(text, view)
       
    begin
        value = text.to_l
    rescue
       
        UI.beep
        puts "Cannot convert #{text} to a Length"
        value = nil
        Sketchup::set_status_text "", SB_VCB_VALUE
    end
    return if !value

	$focus_distance=value
	if value==0
		UI.messagebox ("Camera set to Autofocus")
		model=Sketchup.active_model
		$focal_point=model.set_attribute 'indigo07_settings','fp',"auto"
	else
    	UI.messagebox ("New Focus Distance set to #{$focus_distance}")
	end

           
    self.reset(view)
	Sketchup.active_model.select_tool nil
end


def draw(view)
    if( @ip1.valid? )
        if( @ip1.display? )
            @ip1.draw(view)
            @drawn = true
        end
        
    end
end



def onCancel(flag, view)
    self.reset(view)
end




# Reset the tool back to its initial state
def reset(view)
       
    Sketchup::set_status_text("Select Point on Focal Plane or Enter Focus Distance in VCB", SB_PROMPT)
    
    
    @ip1.clear
    
    
    if( view )
        view.tooltip = nil
        view.invalidate 
    end
    
   
end


end # class FocusTool

def focustool
    Sketchup.active_model.select_tool FocusTool.new
end


##########################################Context menus##################################
def selected_material
	ent = Sketchup.active_model.selection.first
	return nil if ent==nil
	$selected_material=ent.material if ent!=nil
	return nil if not $selected_material
	return $selected_material
end

def selected_component
	ent = Sketchup.active_model.selection.first
	return nil if ent==nil
	return nil if ent.class!=Sketchup::ComponentInstance
	return ent.definition
end

UI.add_context_menu_handler do |menu|
	ent=Sketchup.active_model.selection.first
	mat=ent.material if ent
        if mat
		
            menu.add_separator
		type=(mat.get_attribute 'indigo07_materials','type',"Diffuse").upcase
            preset_menu=menu.add_submenu("SkIndigo Preset \"#{mat.name}\" (#{type})")
		preset_menu.add_item("Edit Material") {(SU2IN.assign_properties_to_material(mat))}
		preset_menu.add_item("Glass") { (SU2IN.apply_glass(mat)) }
		preset_menu.add_item("Frosted Glass") { (SU2IN.apply_frosted_glass(mat)) }
		preset_menu.add_item("Mirror") { (SU2IN.apply_mirror(mat)) }
		preset_menu.add_item("Ceramic") { (SU2IN.apply_ceramic(mat)) }
		preset_menu.add_item("Dull Plastic") { (SU2IN.apply_dull_plastic(mat)) }
		preset_menu.add_item("Shiny Plastic") { (SU2IN.apply_shiny_plastic(mat)) }
		preset_menu.add_item("Leather") { (SU2IN.apply_leather(mat)) }
		preset_menu.add_item("Car Paint") { (SU2IN.apply_car_paint(mat)) }
		preset_menu.add_item("Aluminum") { (SU2IN.apply_Al(mat)) }
		preset_menu.add_item("Chrome") { (SU2IN.apply_Cr(mat)) }
		preset_menu.add_item("Copper") { (SU2IN.apply_Cu(mat)) }
		preset_menu.add_item("Gold") { (SU2IN.apply_Au(mat)) }
		preset_menu.add_item("Silver") { (SU2IN.apply_Ag(mat)) }
		preset_menu.add_item("Curtains") { (SU2IN.apply_curtains(mat)) }
		preset_menu.add_item("Lampshade") { (SU2IN.apply_lampshade(mat)) }
		preset_menu.add_item("100W Incandescent") { (SU2IN.apply_100W(mat)) }
		preset_menu.add_item("60W Incandescent") { (SU2IN.apply_60W(mat)) }
		preset_menu.add_item("7W Compact Fluorescent (warm)") { (SU2IN.apply_7WCFW(mat)) }
		preset_menu.add_item("34W Tubular Fluorescent (cold)") { (SU2IN.apply_34WTFC(mat)) }
		preset_menu.add_item("34W Tubular Fluorescent (mid)") { (SU2IN.apply_34WTFM(mat)) }
		preset_menu.add_item("34W Tubular Fluorescent (warm)") { (SU2IN.apply_34WTFW(mat)) }
		#preset_menu.add_item("Lithium") { (SU2IN.apply_Li(mat)) }
		#preset_menu.add_item("Nickel") { (SU2IN.apply_Ni(mat)) }
		#preset_menu.add_item("Platinum") { (SU2IN.apply_Pt(mat)) }
		
        end
   
end

UI.add_context_menu_handler do |menu|
	ent=Sketchup.active_model.selection.first
	if ent.class==Sketchup::ComponentInstance
		e=ent.definition
	end
	
	if e
		instanced=e.get_attribute 'indigo07_settings','inst',false
		menu.add_separator if (ent.material==nil)
		if not instanced
			menu.add_item("SkIndigo \"#{e.name}\" Enable Instancing") {(e.set_attribute 'indigo07_settings','inst',true)}
		else
			menu.add_item("SkIndigo \"#{e.name}\" Disable Instancing") {(e.set_attribute 'indigo07_settings','inst',false)}
		end
	end
		
end


UI.add_context_menu_handler do |menu|
	
        if $ss_exported and ($g_su2in_model_name!="")
		ent=Sketchup.active_model.selection.first
		if ent
			menu.add_separator if ent.material==nil
		else
			menu.add_separator
		end	
            menu.add_item("SkIndigo Quick Export") {(SU2IN.quick_export)}
		
        end
   
end




####################Advanced user functions#####################
class Indigo

def Indigo::adv

$indigo_path=SU2IN.get_indigo_path
return if $indigo_path==nil
inifile_path=File.dirname($indigo_path)
inifile_path+="\/inifile.txt"
if FileTest.exist?(inifile_path)
	UI.openURL(inifile_path)
else
	puts "inifile.txt not found"
end

end

def Indigo::ren

$indigo_path=SU2IN.get_indigo_path
return if $indigo_path==nil
renders_path=File.dirname($indigo_path)
renders_path+="\/renders"
if FileTest.exist?(renders_path)
	UI.openURL(renders_path) 
else
	puts "renders directory not found"
end

end

def Indigo::igs

return if $g_su2in_export_file==nil
if $mat_path!=""
	if FileTest.exist?($mat_path)
		UI.openURL($mat_path)
	else
		puts "igs not found"
	end 
else
	if FileTest.exist?($g_su2in_export_file)
		UI.openURL($g_su2in_export_file) 
	else
		puts "igs not found"
	end
end

end

def Indigo::tm

$indigo_path=SU2IN.get_indigo_path
return if $indigo_path==nil
tm_path=File.dirname($indigo_path)
tm_path+="\/IndigoToneMapper.exe"
if FileTest.exist?(tm_path)
	UI.openURL(tm_path) 
else
	puts "IndigoToneMapper.exe not found"
end

end

def Indigo::web

UI.openURL("http:\/\/www2.indigorenderer.com\/joomla\/")

end

def Indigo::mdb

UI.openURL("http:\/\/www.elqx.com\/indmatdb\/")

end

def Indigo::res

$indigo_path=SU2IN.get_indigo_path
return if $indigo_path==nil

igi_file=UI.openpanel("Locate Indigo Image File", "" , "*.igi")
return if igi_file==nil
igs_file=UI.openpanel("Locate Indigo Scene File","","*.igs")
return if igs_file==nil

batch_file_path=File.dirname($indigo_path)+"\/resume.bat"
batch=File.new(batch_file_path,"w")

batch.puts "start \/low indigo.exe -r \"#{igi_file}\" \"#{igs_file}\"" if $threads=="Low Priority"
batch.puts "start indigo.exe -r \"#{igi_file}\" \"#{igs_file}\"" if $threads!="Low Priority"
batch.close

UI.openURL(batch_file_path)

end

end #class Indigo end

class SkindigoMatObserver <  Sketchup::MaterialsObserver
 	def onMaterialSetCurrent(mats,mat)
		
		#$selected_material=mat
		if $start
			finish=Time.new
			if (finish-$start)<0.25
				SU2IN.assign_properties_to_material(mat)
			else
				$start=Time.new
			end
		else
			$start=Time.new
		end

		

	end

	
	
end

class SkindigoAppObserver <  Sketchup::AppObserver
 	def onOpenModel(m)

		smo=nil
		smo=SkindigoMatObserver.new
		status=Sketchup.active_model.materials.add_observer(smo)
		$g_su2in_loaded_entities=nil if $g_su2in_loaded_entities

	end

	def onNewModel(m)
				
		smo=nil
		smo=SkindigoMatObserver.new
		Sketchup.active_model.materials.add_observer(smo)
		$g_su2in_loaded_entities=nil if $g_su2in_loaded_entities
	end
	
	def onQuit(m)
				
		Sketchup.active_model.materials.remove_observer(smo) if smo
		smo=nil if smo
		

	end
end


##################UV Class to Manage UV Sets####################3
class UVS


def UVS::store_uvs(uvset)

face=0
case uvset
	when 0
		key="uv0"
	when 1
		key="uv1"
	when 2
		key="uv2"
	when 3
		key="uv3"
end

Sketchup.active_model.start_operation "Save UV set"

for et in $g_su2in_loaded_entities
	
	if (et[0].valid?) and (et[0].class==Sketchup::Face)
		pos=[]
		polymesh=et[0].mesh 5
		uvs=polymesh.uvs 5
		polymesh.transform! et[1]
		
		for p in (1..4)
			point=polymesh.point_at(p)
			if point
				pos.push(point) 
				
			end
		end
		et[0].set_attribute 'uvs','points',pos
		et[0].set_attribute 'uvs',key,uvs
	
	
	face=face+1
	Sketchup.set_status_text("Stored uvs for #{face} faces.")
	end
		
end

UI.messagebox "UVs saved for #{face} faces."

Sketchup.active_model.commit_operation

end #function

##########################################load entities#####################
def UVS::load_mat_entities(mat)

Sketchup.active_model.start_operation "Load entities"
$ents=0
$g_su2in_loaded_entities=[]
entities=Sketchup.active_model.entities
load_entities(mat,entities,Geom::Transformation.new)

UI.messagebox "Loaded #{$ents} entities currently assigned material \"#{mat.display_name}\"."

Sketchup.active_model.commit_operation

end


##########################################load entities recursive######################
def UVS::load_entities(mat,object,trans)


if object.class == Sketchup::ComponentInstance
	entity_list=object.definition.entities
elsif object.class == Sketchup::Group
	entity_list=object.entities
else
	entity_list=object
end

Sketchup.set_status_text("Collected #{$ents} entities assigned material #{mat.display_name}")

for e in entity_list
		if (e.class == Sketchup::Group)
			
			if e.material == mat
				$g_su2in_loaded_entities.push([e,trans])
				Sketchup.set_status_text("Collected #{$ents=$ents+1} entities assigned material #{mat.display_name}")
			end
			UVS.load_entities(mat,e,trans*e.transformation)
			
		end
		if (e.class == Sketchup::ComponentInstance)
			if e.material == mat
				$g_su2in_loaded_entities.push([e,trans])
				Sketchup.set_status_text("Collected #{$ents=$ents+1} entities assigned material #{mat.display_name}")
			
			end
			UVS.load_entities(mat,e,trans*e.transformation)
							
		end
		if (e.class == Sketchup::Face)
			
			if e.material == mat
				$g_su2in_loaded_entities.push([e,trans])
				Sketchup.set_status_text("Collected #{$ents=$ents+1} entities assigned material #{mat.display_name}")
					
			end
			
		end
end

return nil

end #function

####################################switch the texture map#########################3

def UVS::switch_material(mat)

materials = Sketchup.active_model.materials
names = materials.collect {|m| m.name}
translatednames = materials.collect {|m| m.display_name}
    
# Display a dialog to pick a material

    prompts = ["Material Name"]
    values=[materials[0].display_name]
    enums = [translatednames.sort.join("|")]
    results = inputbox prompts, values, enums,("Assign all loaded entities to:")
    return if not results
	
current_mat=results[0]

# Get the selected Material

index = translatednames.index(current_mat)
material = index ? materials[names[index]] : nil

if( not material )
       UI.messagebox ("Could not find Material named") + " #{results[0]}"
       return
end

Sketchup.active_model.start_operation "Switch Material"
for et in $g_su2in_loaded_entities
	if et[0].valid?
		et[0].material=material 
	end
end

if material.texture!=nil
	prompts=["Select UV index"]
	values=[0]
	enums=[[0,1,2,3].join("|")]
	result = inputbox prompts, values, enums,("Select UV set")
	if result
		position_map(result[0])
	end
end

Sketchup.active_model.commit_operation

end


################################Function to position the material according to stored uvset########
def UVS::position_map(set)

Sketchup.active_model.start_operation "Load UV set"

$ents=0
face=0
 
case set
	
	when 0
		key="uv0"
	when 1
		key="uv1"
	when 2
		key="uv2"
	when 3
		key="uv3"
end

begin

for et in $g_su2in_loaded_entities
	if (et[0].valid?) and (et[0].class==Sketchup::Face)
		uvs=et[0].get_attribute 'uvs',key
		
		if uvs
			points=et[0].get_attribute 'uvs','points'
			pos=[]
			for p in (0..3)
				if points[p]
					pos.push(points[p]) 
					pos.push(uvs[p])
				end
			end
			#UI.messagebox et[0]
			et[0].position_material et[0].material, pos, true
			face=face+1
			Sketchup.set_status_text("Loaded uvs for #{face} faces")
		end
	end
end
rescue
UI.messagebox "Unable to load UVs."
end

UI.messagebox ("Loaded uvs for #{face} faces.")

Sketchup.active_model.commit_operation

end


########################Assign uv sets to material################
def UVS::assign_uvs(mat)

if mat.alpha<1.0
	type=mat.get_attribute 'indigo07_materials','type',"Specular"
else
	type=mat.get_attribute 'indigo07_materials','type',"Diffuse" 
end

uvsets_choices=["Default","0","1","2","3"].join("|")
case type

when "Diffuse"
	prompts=["Albedo (Diffuse)","Bump","Clip"]
	values=[(mat.get_attribute 'uvsets','albedo',"Default"),(mat.get_attribute 'uvsets','bump',"Default"),(mat.get_attribute 'uvsets','clip',"Default")]
	enums=[uvsets_choices,uvsets_choices,uvsets_choices]
	results=inputbox prompts,values,enums,"Assign UV sets for \"#{mat.display_name}\""
	return if not results
	mat.set_attribute 'uvsets','albedo',results[0]
	mat.set_attribute 'uvsets','bump',results[1]
	mat.set_attribute 'uvsets','clip',results[2]
when "Phong"
	prompts=["Albedo (Diffuse)","Bump","Exponent","Clip"]
	values=[(mat.get_attribute 'uvsets','albedo',"Default"),(mat.get_attribute 'uvsets','bump',"Default"),(mat.get_attribute 'uvsets','exponent',"Default"),(mat.get_attribute 'uvsets','clip',"Default")]
	enums=[uvsets_choices,uvsets_choices,uvsets_choices,uvsets_choices]
	results=inputbox prompts,values,enums,"Assign UV sets for \"#{mat.display_name}\""
	return if not results
	mat.set_attribute 'uvsets','albedo',results[0]
	mat.set_attribute 'uvsets','bump',results[1]
	mat.set_attribute 'uvsets','exponent',results[2]
	mat.set_attribute 'uvsets','clip',results[3]
when "Au","Ag","Al","Cr","Cu","Ti","W","Li","Ni","Pt"
	prompts=["Bump","Exponent","Clip"]
	values=[(mat.get_attribute 'uvsets','bump',"Default"),(mat.get_attribute 'uvsets','exponent',"Default"),(mat.get_attribute 'uvsets','clip',"Default")]
	enums=[uvsets_choices,uvsets_choices,uvsets_choices]
	results=inputbox prompts,values,enums,"Assign UV sets for \"#{mat.display_name}\""
	return if not results
	mat.set_attribute 'uvsets','bump',results[0]
	mat.set_attribute 'uvsets','exponent',results[1]
	mat.set_attribute 'uvsets','clip',results[2]
when "Specular"
	prompts=["Bump","Clip"]
	values=[(mat.get_attribute 'uvsets','bump',"Default"),(mat.get_attribute 'uvsets','clip',"Default")]
	enums=[uvsets_choices,uvsets_choices]
	results=inputbox prompts,values,enums,"Assign UV sets for \"#{mat.display_name}\""
	return if not results
	mat.set_attribute 'uvsets','bump',results[0]
	mat.set_attribute 'uvsets','clip',results[1]
when "Glossy Transparent"
	prompts=["Bump","Exponent","Clip"]
	values=[(mat.get_attribute 'uvsets','bump',"Default"),(mat.get_attribute 'uvsets','exponent',"Default"),(mat.get_attribute 'uvsets','clip',"Default")]
	enums=[uvsets_choices,uvsets_choices,uvsets_choices]
	results=inputbox prompts,values,enums,"Assign UV sets for \"#{mat.display_name}\""
	return if not results
	mat.set_attribute 'uvsets','bump',results[0]
	mat.set_attribute 'uvsets','exponent',results[1]
	mat.set_attribute 'uvsets','clip',results[2]
when "Blend"
	prompts=["Blend","Clip"]
	values=[(mat.get_attribute 'uvsets','blend',"Default"),(mat.get_attribute 'uvsets','clip',"Default")]
	enums=[uvsets_choices,uvsets_choices]
	results=inputbox prompts,values,enums,"Assign UV sets for \"#{mat.display_name}\""
	return if not results
	mat.set_attribute 'uvsets','blend',results[0]
	mat.set_attribute 'uvsets','clip',results[1]
when "Blackbody Light"
	UI.messagebox "Cannot assign UV sets to #{type} materials."
	return
when "RGB Light"
	UI.messagebox "Cannot assign UV sets to #{type} materials."
	return
When "Thin Glass"
	UI.messagebox "Cannot assign UV sets to #{type} materials."
	return
end

end

########################Purge UV sets################
def UVS::purge_uvs

result=UI.messagebox "Would you like to purge all stored UV sets for the currently loaded entities?",MB_YESNO
return if result!=6

Sketchup.active_model.start_operation "Purge UV sets"

faces=0

for et in $g_su2in_loaded_entities
	if (et[0].valid?) and (et[0].class==Sketchup::Face)
		et[0].delete_attribute 'uvs'
		faces=faces+1
	end
end  


UI.messagebox "Purged UV sets for #{faces} faces."

Sketchup.active_model.commit_operation

end





####################################Explode Distorted Textures#########################
def UVS::explode_distorted_texture

face=Sketchup.active_model.selection.first
return if not face
return if face.class!=Sketchup::Face

Sketchup.active_model.start_operation "Explode Texture"
mat=face.material
materials=Sketchup.active_model.materials
new_mat=materials.add(mat.name)
new_name=new_mat.display_name+".jpg"

tw=Sketchup.create_texture_writer

if (mat.respond_to?(:texture) and mat.texture !=nil)
	status=tw.load face,true
else
	return nil
end

texture_path=UI.savepanel("Save Distorted Texture", "" ,new_name )

return if texture_path==nil		
if texture_path==texture_path.split(".")[0] #no extension given
	texture_path+=".jpg"
end


arr=[]

uvh=face.get_UVHelper true,false,tw

for p in (0..3)
	vert=face.vertices[p]
	pos=vert.position
	uvq = uvh.get_front_UVQ(pos)
	arr.push(pos)
	arr.push(uvq)
end

tw.write(face,true,texture_path)
new_mat.texture=(texture_path) if new_mat
face.position_material(new_mat,arr,true)

dict="indigo07_materials"
att=["type","exp","fres","ior","cau","precedence","sss","sssval","power","ole","bump_map","bump_value","temp","no_smooth","exp_map","expmap_value","clip_map"]

for a in att
	new_mat.set_attribute dict,a,(mat.get_attribute dict,a)
end

dict="indigo_blend"
att=[]
att=["matA","matB","blendfactor"]

for a in att
	new_mat.set_attribute dict,a,(mat.get_attribute dict,a)
end

new_mat.alpha=mat.alpha

Sketchup.active_model.commit_operation
UI.messagebox "Texture was exploded and applied to the selected face."

end


end #Class end


##############################UV Manager Context menu##########################

UI.add_context_menu_handler do |menu|
	mat=selected_material
	
	
		menu.add_separator
		uv_menu=menu.add_submenu("SkIndigo UV Mapping") if (mat or $g_su2in_loaded_entities)
		uv_menu.add_item("Load entities assigned to material \"#{mat.display_name}\"") {UVS.load_mat_entities(mat)} if mat
		uv_menu.add_item("Assign UV sets for \"#{mat.display_name}\"") {UVS.assign_uvs(mat)} if mat
		uv_menu.add_item("Explode distorted texture") {(UVS.explode_distorted_texture)} if (mat and mat.texture)
		uv_menu.add_item("Switch Material") {UVS.switch_material(mat)} if $g_su2in_loaded_entities
		uv_menu.add_separator if $g_su2in_loaded_entities
		uv_menu.add_item("Save uvs to index 0") {UVS.store_uvs(0)} if $g_su2in_loaded_entities
		uv_menu.add_item("Save uvs to index 1") {UVS.store_uvs(1)} if $g_su2in_loaded_entities
		uv_menu.add_item("Save uvs to index 2") {UVS.store_uvs(2)} if $g_su2in_loaded_entities
		uv_menu.add_item("Save uvs to index 3") {UVS.store_uvs(3)} if $g_su2in_loaded_entities
		uv_menu.add_item("Purge all UV sets") {UVS.purge_uvs} if $g_su2in_loaded_entities
		uv_menu.add_separator if $g_su2in_loaded_entities
		uv_menu.add_item("Load uvs for index 0") {UVS.position_map(0)} if $g_su2in_loaded_entities
		uv_menu.add_item("Load uvs for index 1") {UVS.position_map(1)} if $g_su2in_loaded_entities
		uv_menu.add_item("Load uvs for index 2") {UVS.position_map(2)} if $g_su2in_loaded_entities
		uv_menu.add_item("Load uvs for index 3") {UVS.position_map(3)} if $g_su2in_loaded_entities
	

		
end

# ---------------------------------------- Menu item
if( not file_loaded?("SkIndigo_08.rb") )

	main_menu = UI.menu("Plugins").add_submenu("SkIndigo")
	main_menu.add_item("Export Model") { (SU2IN.export) }
	main_menu.add_item("Material Editor") { (SU2IN.assign_properties_to_material(nil)) }
	main_menu.add_item("Query Material Types") { (SU2IN.mat_query) }
	main_menu.add_item("Render Settings") { (SU2IN.render_settings)}
	main_menu.add_item("Set Camera Focus") {(focustool)}
	main_menu.add_item("Set Environment Map") {(SU2IN.load_exr)}
	main_menu.add_separator
	main_menu.add_item("About SkIndigo 0.8 beta2") { (SU2IN.about) }
	
	Sketchup.add_observer(SkindigoAppObserver.new)

	smo=SkindigoMatObserver.new
	Sketchup.active_model.materials.add_observer(smo)
	
	#SU2IN.reset_global_variables	
	
end 

#-----------------------------------------------------------
file_loaded("SkIndigo_08.rb")

